from _typeshed import Incomplete
from weakref import ReferenceType

from reportlab.lib.colors import Color
from reportlab.pdfbase.pdfdoc import PDFDictionary, PDFObject, PDFStream

__all__ = ("AcroForm",)

visibilities: dict[str, int]
orientations: dict[str, list[Incomplete]]
fieldFlagValues: dict[str, int]
annotationFlagValues: dict[str, int]

def bsPDF(borderWidth: int, borderStyle: str, dashLen) -> PDFDictionary: ...
def escPDF(s) -> str: ...
def makeFlags(s: int | str, d: dict[str, int] = ...) -> int: ...

class PDFFromString(PDFObject):
    def __init__(self, s: str | bytes) -> None: ...
    def format(self, document) -> bytes: ...

class RadioGroup(PDFObject):
    TU: Incomplete
    Ff: int
    kids: list[Incomplete]
    T: Incomplete
    V: Incomplete
    def __init__(self, name, tooltip: str = "", fieldFlags: str = "noToggleToOff required radio") -> None: ...
    def format(self, doc) -> bytes: ...

class AcroForm(PDFObject):
    formFontNames: dict[str, str]
    referenceMap: dict[Incomplete, Incomplete]
    fonts: dict[str, str]
    fields: list[Incomplete]
    sigFlags: Incomplete
    extras: dict[Incomplete, Incomplete]
    def __init__(self, canv, **kwds) -> None: ...
    @property
    def useDefault(self) -> object: ...
    @property
    def canv(self) -> ReferenceType[Incomplete]: ...
    def fontRef(self, f) -> str: ...
    def format(self, doc) -> bytes: ...
    def colorTuple(self, c): ...
    def streamFillColor(self, c) -> str: ...
    def streamStrokeColor(self, c) -> str: ...
    def checkboxAP(
        self,
        key,
        value,
        buttonStyle: str = "circle",
        shape: str = "square",
        fillColor=None,
        borderColor=None,
        textColor=None,
        borderWidth: int | None = None,
        borderStyle: str = "solid",
        size: int = 20,
        dashLen: int = 3,
    ) -> PDFStream: ...
    @staticmethod
    def circleArcStream(size, r, arcs=(0, 1, 2, 3), rotated: bool = False) -> str: ...
    def zdMark(self, c, size, ds, iFontName) -> str: ...
    def getRef(self, obj): ...
    def getRefStr(self, obj) -> str: ...
    def setDefault(self, name: str, value: Color | float | None) -> None: ...
    def getDefaults(
        self, textColor, borderColor, fillColor, borderWidth
    ) -> tuple[Incomplete, Incomplete, Incomplete, Incomplete]: ...
    @staticmethod
    def varyColors(key, t, b, f) -> tuple[Incomplete, Incomplete, Incomplete]: ...
    def checkForceBorder(
        self, x, y, width, height, forceBorder, shape, borderStyle, borderWidth, borderColor, fillColor
    ) -> None: ...
    def checkbox(
        self,
        checked: bool = False,
        buttonStyle: str = "check",
        shape: str = "square",
        fillColor=...,
        borderColor=...,
        textColor=...,
        borderWidth: int = ...,
        borderStyle: str = "solid",
        size: int = 20,
        x: int = 0,
        y: int = 0,
        tooltip=None,
        name=None,
        annotationFlags: str = "print",
        fieldFlags: str = "required",
        forceBorder: bool = False,
        relative: bool = False,
        dashLen: int = 3,
    ) -> None: ...
    def radio(
        self,
        value=None,
        selected: bool = False,
        buttonStyle: str = "circle",
        shape: str = "circle",
        fillColor=...,
        borderColor=...,
        textColor=...,
        borderWidth: int = ...,
        borderStyle: str = "solid",
        size: int = 20,
        x: int = 0,
        y: int = 0,
        tooltip=None,
        name=None,
        annotationFlags: str = "print",
        fieldFlags: str = "noToggleToOff required radio",
        forceBorder: bool = False,
        relative: bool = False,
        dashLen: int = 3,
    ) -> None: ...
    def makeStream(self, width, height, stream, **D) -> PDFStream: ...
    def txAP(
        self,
        key,
        value,
        iFontName,
        rFontName,
        fontSize,
        shape: str = "square",
        fillColor=None,
        borderColor=None,
        textColor=None,
        borderWidth: int | None = None,
        borderStyle: str = "solid",
        width: int = 120,
        height: int = 36,
        dashLen: int = 3,
        wkind: str = "textfield",
        labels=[],
        I=[],
        sel_bg: str = "0.600006 0.756866 0.854904 rg",
        sel_fg: str = "0 g",
    ) -> PDFStream: ...
    def makeFont(self, fontName: str | None) -> tuple[str, str]: ...
    def textfield(
        self,
        value: str = "",
        fillColor=...,
        borderColor=...,
        textColor=...,
        borderWidth: int = ...,
        borderStyle: str = "solid",
        width: int = 120,
        height: int = 36,
        x: int = 0,
        y: int = 0,
        tooltip=None,
        name=None,
        annotationFlags: str = "print",
        fieldFlags: str = "",
        forceBorder: bool = False,
        relative: bool = False,
        maxlen: int = 100,
        fontName: str | None = None,
        fontSize=None,
        dashLen: int = 3,
    ) -> None: ...
    def listbox(
        self,
        value: str = "",
        fillColor=...,
        borderColor=...,
        textColor=...,
        borderWidth: int = ...,
        borderStyle: str = "solid",
        width: int = 120,
        height: int = 36,
        x: int = 0,
        y: int = 0,
        tooltip=None,
        name=None,
        annotationFlags: str = "print",
        fieldFlags: str = "",
        forceBorder: bool = False,
        relative: bool = False,
        fontName: str | None = None,
        fontSize=None,
        dashLen: int = 3,
        maxlen=None,
        options=[],
    ) -> None: ...
    def choice(
        self,
        value: str = "",
        fillColor=...,
        borderColor=...,
        textColor=...,
        borderWidth: int = ...,
        borderStyle: str = "solid",
        width: int = 120,
        height: int = 36,
        x: int = 0,
        y: int = 0,
        tooltip=None,
        name=None,
        annotationFlags: str = "print",
        fieldFlags: str = "combo",
        forceBorder: bool = False,
        relative: bool = False,
        fontName: str | None = None,
        fontSize=None,
        dashLen: int = 3,
        maxlen=None,
        options=[],
    ) -> None: ...
    def checkboxRelative(
        self,
        *,
        checked: bool = False,
        buttonStyle: str = "check",
        shape: str = "square",
        fillColor=...,
        borderColor=...,
        textColor=...,
        borderWidth: int = ...,
        borderStyle: str = "solid",
        size: int = 20,
        x: int = 0,
        y: int = 0,
        tooltip=None,
        name=None,
        annotationFlags: str = "print",
        fieldFlags: str = "required",
        forceBorder: bool = False,
        dashLen: int = 3,
    ) -> None: ...
    def radioRelative(
        self,
        *,
        value=None,
        selected: bool = False,
        buttonStyle: str = "circle",
        shape: str = "circle",
        fillColor=...,
        borderColor=...,
        textColor=...,
        borderWidth: int = ...,
        borderStyle: str = "solid",
        size: int = 20,
        x: int = 0,
        y: int = 0,
        tooltip=None,
        name=None,
        annotationFlags: str = "print",
        fieldFlags: str = "noToggleToOff required radio",
        forceBorder: bool = False,
        dashLen: int = 3,
    ) -> None: ...
    def textfieldRelative(
        self,
        *,
        value: str = "",
        fillColor=...,
        borderColor=...,
        textColor=...,
        borderWidth: int = ...,
        borderStyle: str = "solid",
        width: int = 120,
        height: int = 36,
        x: int = 0,
        y: int = 0,
        tooltip=None,
        name=None,
        annotationFlags: str = "print",
        fieldFlags: str = "",
        forceBorder: bool = False,
        maxlen: int = 100,
        fontName: str | None = None,
        fontSize=None,
        dashLen: int = 3,
    ) -> None: ...
    def listboxRelative(
        self,
        *,
        value: str = "",
        fillColor=...,
        borderColor=...,
        textColor=...,
        borderWidth: int = ...,
        borderStyle: str = "solid",
        width: int = 120,
        height: int = 36,
        x: int = 0,
        y: int = 0,
        tooltip=None,
        name=None,
        annotationFlags: str = "print",
        fieldFlags: str = "",
        forceBorder: bool = False,
        maxlen: int = 100,
        fontName: str | None = None,
        fontSize=None,
        dashLen: int = 3,
    ) -> None: ...
    def choiceRelative(
        self,
        *,
        value: str = "",
        fillColor=...,
        borderColor=...,
        textColor=...,
        borderWidth: int = ...,
        borderStyle: str = "solid",
        width: int = 120,
        height: int = 36,
        x: int = 0,
        y: int = 0,
        tooltip=None,
        name=None,
        annotationFlags: str = "print",
        fieldFlags: str = "",
        forceBorder: bool = False,
        maxlen: int = 100,
        fontName: str | None = None,
        fontSize=None,
        dashLen: int = 3,
    ) -> None: ...
    @property
    def encRefStr(self) -> str: ...

class CBMark:
    opNames: list[str]
    opCount: tuple[int, ...]
    ops: Incomplete
    xmin: Incomplete
    ymin: Incomplete
    xmax: Incomplete
    ymax: Incomplete
    points: Incomplete
    slack: Incomplete
    def __init__(self, ops, points, bounds, slack: float = 0.05) -> None: ...
    def scaledRender(self, size, ds: int = 0) -> str: ...



from ..base_client import BaseApp, OAuth1Mixin, OAuth2Mixin, OpenIDMixin
from ..requests_client import OAuth1Session, OAuth2Session

class DjangoAppMixin:
    def save_authorize_data(self, request, **kwargs) -> None: ...
    def authorize_redirect(self, request, redirect_uri=None, **kwargs): ...

class DjangoOAuth1App(DjangoAppMixin, OAuth1Mixin, BaseApp):
    client_cls = OAuth1Session
    def authorize_access_token(self, request, **kwargs): ...

class DjangoOAuth2App(DjangoAppMixin, OAuth2Mixin, OpenIDMixin, BaseApp):
    client_cls = OAuth2Session
    def authorize_access_token(self, request, **kwargs): ...



import sys
from _typeshed import SupportsRichComparisonT
from types import GenericAlias
from typing import Any, Generic, TypeVar

from .mixins import _LoopBoundMixin

class QueueEmpty(Exception): ...
class QueueFull(Exception): ...

# Keep asyncio.__all__ updated with any changes to __all__ here
if sys.version_info >= (3, 13):
    __all__ = ("Queue", "PriorityQueue", "LifoQueue", "QueueFull", "QueueEmpty", "QueueShutDown")

else:
    __all__ = ("Queue", "PriorityQueue", "LifoQueue", "QueueFull", "QueueEmpty")

_T = TypeVar("_T")

if sys.version_info >= (3, 13):
    class QueueShutDown(Exception): ...

class Queue(_LoopBoundMixin, Generic[_T]):
    def __init__(self, maxsize: int = 0) -> None: ...
    def _init(self, maxsize: int) -> None: ...
    def _get(self) -> _T: ...
    def _put(self, item: _T) -> None: ...
    def _format(self) -> str: ...
    def qsize(self) -> int: ...
    @property
    def maxsize(self) -> int: ...
    def empty(self) -> bool: ...
    def full(self) -> bool: ...
    async def put(self, item: _T) -> None: ...
    def put_nowait(self, item: _T) -> None: ...
    async def get(self) -> _T: ...
    def get_nowait(self) -> _T: ...
    async def join(self) -> None: ...
    def task_done(self) -> None: ...
    def __class_getitem__(cls, type: Any, /) -> GenericAlias: ...
    if sys.version_info >= (3, 13):
        def shutdown(self, immediate: bool = False) -> None: ...

class PriorityQueue(Queue[SupportsRichComparisonT]): ...
class LifoQueue(Queue[_T]): ...



from _typeshed import Incomplete

from authlib.oauth2 import ResourceProtector as _ResourceProtector
from authlib.oauth2.rfc6750 import BearerTokenValidator as _BearerTokenValidator

class ResourceProtector(_ResourceProtector):
    def acquire_token(self, request, scopes=None, **kwargs): ...
    def __call__(self, scopes=None, optional=False, **kwargs): ...

class BearerTokenValidator(_BearerTokenValidator):
    token_model: Incomplete
    def __init__(self, token_model, realm=None, **extra_attributes): ...
    def authenticate_token(self, token_string): ...

def return_error_response(error): ...



from typing_extensions import deprecated

@deprecated("The `nturl2path` module is deprecated since Python 3.14.")
def url2pathname(url: str) -> str: ...
@deprecated("The `nturl2path` module is deprecated since Python 3.14.")
def pathname2url(p: str) -> str: ...



from _typeshed import Incomplete
from collections.abc import Generator
from importlib.metadata import EntryPoints
from typing import Final

from pygments.filter import Filter
from pygments.formatter import Formatter
from pygments.lexer import Lexer
from pygments.style import Style

LEXER_ENTRY_POINT: Final = "pygments.lexers"
FORMATTER_ENTRY_POINT: Final = "pygments.formatters"
STYLE_ENTRY_POINT: Final = "pygments.styles"
FILTER_ENTRY_POINT: Final = "pygments.filters"

def iter_entry_points(group_name: str) -> EntryPoints: ...
def find_plugin_lexers() -> Generator[type[Lexer]]: ...
def find_plugin_formatters() -> Generator[tuple[str, type[Formatter[Incomplete]]]]: ...
def find_plugin_styles() -> Generator[tuple[str, type[Style]]]: ...
def find_plugin_filters() -> Generator[tuple[str, type[Filter]]]: ...



from _typeshed import Incomplete, ReadableBuffer
from collections.abc import Iterable
from enum import Enum
from io import BytesIO

from gunicorn.http2.connection import HTTP2ServerConnection

class StreamState(Enum):
    IDLE = 1
    RESERVED_LOCAL = 2
    RESERVED_REMOTE = 3
    OPEN = 4
    HALF_CLOSED_LOCAL = 5
    HALF_CLOSED_REMOTE = 6
    CLOSED = 7

class HTTP2Stream:
    stream_id: int
    connection: HTTP2ServerConnection
    state: StreamState
    request_headers: list[tuple[str, Incomplete]]
    request_body: BytesIO
    request_complete: bool
    response_started: bool
    response_headers_sent: bool
    response_complete: bool
    window_size: int
    trailers: list[tuple[str, Incomplete]] | None
    response_trailers: list[tuple[str, Incomplete]] | None
    priority_weight: int
    priority_depends_on: int
    priority_exclusive: bool

    def __init__(self, stream_id: int, connection: HTTP2ServerConnection) -> None: ...
    @property
    def is_client_stream(self) -> bool: ...
    @property
    def is_server_stream(self) -> bool: ...
    @property
    def can_receive(self) -> bool: ...
    @property
    def can_send(self) -> bool: ...
    def receive_headers(self, headers: Iterable[tuple[str, Incomplete]], end_stream: bool | None = False) -> None: ...
    def receive_data(self, data: ReadableBuffer, end_stream: bool | None = False) -> None: ...
    def receive_trailers(self, trailers: list[tuple[str, Incomplete]]) -> None: ...
    def send_headers(self, headers: Iterable[tuple[str, Incomplete]], end_stream: bool | None = False) -> None: ...
    def send_data(self, data: ReadableBuffer, end_stream: bool | None = False) -> None: ...
    def send_trailers(self, trailers: list[tuple[str, Incomplete]]) -> None: ...
    def reset(self, error_code: int = 0x8) -> None: ...
    def close(self) -> None: ...
    def update_priority(
        self, weight: int | None = None, depends_on: int | None = None, exclusive: bool | None = None
    ) -> None: ...
    def get_request_body(self) -> bytes: ...
    async def read_body_chunk(self) -> bytes | None: ...
    def get_pseudo_headers(self) -> dict[str, Incomplete]: ...
    def get_regular_headers(self) -> list[tuple[str, Incomplete]]: ...

__all__ = ["HTTP2Stream", "StreamState"]



import re
from _typeshed import Incomplete
from collections import OrderedDict
from collections.abc import Callable, Iterable, Iterator, Set as AbstractSet
from datetime import date, datetime, tzinfo
from typing import Any, ClassVar, Final, Literal, NamedTuple, TypeAlias, TypeVar, overload, type_check_only

from dateparser.conf import Settings
from dateparser.languages.loader import LocaleDataLoader
from dateparser.languages.locale import Locale

_DateT = TypeVar("_DateT", bound=date)

_DetectLanguagesFunction: TypeAlias = Callable[[str, float], list[str]]
_Period: TypeAlias = Literal["time", "day", "week", "month", "year"]
# Work around attribute and type having the same name.
_Weekday: TypeAlias = Incomplete  # Actually it's dateutil._common.weekday class

@type_check_only
class _DateData(NamedTuple):
    date_obj: datetime | None
    locale: str | None
    period: _Period | None

APOSTROPHE_LOOK_ALIKE_CHARS: Final[list[str]]
RE_NBSP: Final[re.Pattern[str]]
RE_SPACES: Final[re.Pattern[str]]
RE_TRIM_SPACES: Final[re.Pattern[str]]
RE_TRIM_COLONS: Final[re.Pattern[str]]
RE_SANITIZE_SKIP: Final[re.Pattern[str]]
RE_SANITIZE_RUSSIAN: Final[re.Pattern[str]]
RE_SANITIZE_PERIOD: Final[re.Pattern[str]]
RE_SANITIZE_ON: Final[re.Pattern[str]]
RE_SANITIZE_APOSTROPHE: Final[re.Pattern[str]]
RE_SEARCH_TIMESTAMP: Final[re.Pattern[str]]
RE_SANITIZE_CROATIAN: Final[re.Pattern[str]]
RE_SEARCH_NEGATIVE_TIMESTAMP: Final[re.Pattern[str]]

def sanitize_spaces(date_string: str) -> str: ...
def date_range(
    begin: _DateT,
    end: _DateT,
    *,
    dt1: date | None = None,
    dt2: date | None = None,
    years: int = 0,
    months: int = 0,
    days: int = 0,
    leapdays: int = 0,
    weeks: int = 0,
    hours: int = 0,
    minutes: int = 0,
    seconds: int = 0,
    microseconds: int = 0,
    weekday: int | _Weekday | None = None,
    yearday: int | None = None,
    nlyearday: int | None = None,
    microsecond: int | None = None,
) -> Iterator[_DateT]: ...
def get_intersecting_periods(
    low: _DateT, high: _DateT, period: Literal["year", "month", "week", "day", "hour", "minute", "second", "microsecond"] = "day"
) -> Iterator[_DateT]: ...
def sanitize_date(date_string: str) -> str: ...
def get_date_from_timestamp(date_string: str, settings: Settings, negative: bool | None = False) -> datetime | None: ...
def parse_with_formats(date_string: str, date_formats: Iterable[str], settings: Settings) -> DateData: ...

class _DateLocaleParser:
    locale: Locale
    date_string: str
    date_formats: list[str] | tuple[str, ...] | AbstractSet[str] | None
    def __init__(
        self,
        locale: Locale,
        date_string: str,
        date_formats: list[str] | tuple[str, ...] | AbstractSet[str] | None,
        settings: Settings | None = None,
    ) -> None: ...
    @classmethod
    def parse(
        cls,
        locale: Locale,
        date_string: str,
        date_formats: list[str] | tuple[str, ...] | AbstractSet[str] | None = None,
        settings: Settings | None = None,
    ) -> DateData: ...
    def _parse(self) -> DateData | None: ...
    def _try_timestamp(self) -> DateData: ...
    def _try_freshness_parser(self) -> DateData | None: ...
    def _try_absolute_parser(self) -> DateData | None: ...
    def _try_nospaces_parser(self) -> DateData | None: ...
    def _try_parser(self, parse_method: Callable[[str, Settings, tzinfo | None], tuple[datetime, str]]) -> DateData | None: ...
    def _try_given_formats(self) -> DateData | None: ...
    def _get_translated_date(self) -> str: ...
    def _get_translated_date_with_formatting(self) -> str: ...
    def _is_valid_date_data(self, date_data: DateData) -> bool: ...

class DateData:
    date_obj: datetime | None
    locale: str | None
    period: _Period | None
    def __init__(self, *, date_obj: datetime | None = None, period: _Period | None = None, locale: str | None = None) -> None: ...

    @overload
    def __getitem__(self, k: Literal["date_obj"]) -> datetime | None: ...
    @overload
    def __getitem__(self, k: Literal["locale"]) -> str | None: ...
    @overload
    def __getitem__(self, k: Literal["period"]) -> _Period | None: ...

    @overload
    def __setitem__(self, k: Literal["date_obj"], v: datetime) -> None: ...
    @overload
    def __setitem__(self, k: Literal["locale"], v: str) -> None: ...
    @overload
    def __setitem__(self, k: Literal["period"], v: _Period) -> None: ...

class DateDataParser:
    _settings: Settings
    locale_loader: ClassVar[LocaleDataLoader | None]
    try_previous_locales: bool
    use_given_order: bool
    languages: list[str] | None
    locales: list[str] | tuple[str, ...] | AbstractSet[str] | None
    region: str
    detect_languages_function: _DetectLanguagesFunction | None
    previous_locales: OrderedDict[Locale, None]
    def __init__(
        self,
        languages: list[str] | tuple[str, ...] | AbstractSet[str] | None = None,
        locales: list[str] | tuple[str, ...] | AbstractSet[str] | None = None,
        region: str | None = None,
        try_previous_locales: bool = False,
        use_given_order: bool = False,
        settings: Settings | dict[str, Any] | None = None,
        detect_languages_function: _DetectLanguagesFunction | None = None,
    ) -> None: ...
    def get_date_data(
        self, date_string: str, date_formats: list[str] | tuple[str, ...] | AbstractSet[str] | None = None
    ) -> DateData: ...
    def get_date_tuple(
        self, date_string: str, date_formats: list[str] | tuple[str, ...] | AbstractSet[str] | None = None
    ) -> _DateData: ...
    def _get_applicable_locales(self, date_string: str) -> Iterator[Locale]: ...
    def _is_applicable_locale(self, locale: Locale, date_string: str) -> bool: ...
    @classmethod
    def _get_locale_loader(cls: type[DateDataParser]) -> LocaleDataLoader: ...



from functools import cached_property

from authlib.oauth2.rfc6749 import JsonPayload, JsonRequest, OAuth2Payload, OAuth2Request

class FlaskOAuth2Payload(OAuth2Payload):
    def __init__(self, request) -> None: ...
    @property
    def data(self): ...
    @cached_property
    def datalist(self): ...

class FlaskOAuth2Request(OAuth2Request):
    payload: FlaskOAuth2Payload
    def __init__(self, request) -> None: ...
    @property
    def args(self): ...
    @property
    def form(self): ...

class FlaskJsonPayload(JsonPayload):
    def __init__(self, request) -> None: ...
    @property
    def data(self): ...

class FlaskJsonRequest(JsonRequest):
    payload: FlaskJsonPayload
    def __init__(self, request) -> None: ...



class ArcIV:
    def __init__(self, key) -> None: ...
    def reset(self) -> None: ...
    def encode(self, S: str | bytes | list[int]) -> bytes: ...

def encode(text: str | bytes | list[int], key) -> bytes: ...
def decode(text: str | bytes | list[int], key) -> bytes: ...

__all__ = ["ArcIV", "encode", "decode"]



from _typeshed import Incomplete, SupportsWrite
from collections.abc import Iterable
from typing import TypeVar

from pygments.formatter import Formatter
from pygments.lexer import Lexer
from pygments.token import _TokenType

_T = TypeVar("_T", str, bytes)

__all__ = ["LatexFormatter"]

class LatexFormatter(Formatter[_T]):
    docclass: Incomplete
    preamble: Incomplete
    linenos: Incomplete
    linenostart: Incomplete
    linenostep: Incomplete
    verboptions: Incomplete
    nobackground: Incomplete
    commandprefix: Incomplete
    texcomments: Incomplete
    mathescape: Incomplete
    escapeinside: Incomplete
    left: Incomplete
    right: Incomplete
    envname: Incomplete
    def get_style_defs(self, arg: str = ""): ...
    def format_unencoded(self, tokensource: Iterable[tuple[_TokenType, str]], outfile: SupportsWrite[str]) -> None: ...

class LatexEmbeddedLexer(Lexer):
    left: Incomplete
    right: Incomplete
    lang: Incomplete
    def __init__(self, left, right, lang, **options) -> None: ...
    def get_tokens_unprocessed(self, text): ...



from _typeshed import Incomplete

from braintree.error_result import ErrorResult
from braintree.resource_collection import ResourceCollection
from braintree.successful_result import SuccessfulResult
from braintree.transaction import Transaction

class TransactionGateway:
    gateway: Incomplete
    config: Incomplete
    def __init__(self, gateway) -> None: ...
    def adjust_authorization(self, transaction_id, amount) -> SuccessfulResult | ErrorResult | None: ...
    def clone_transaction(self, transaction_id, params) -> SuccessfulResult | ErrorResult | None: ...
    def cancel_release(self, transaction_id) -> SuccessfulResult | ErrorResult | None: ...
    def create(self, params) -> SuccessfulResult | ErrorResult | None: ...
    def credit(self, params) -> SuccessfulResult | ErrorResult | None: ...
    def find(self, transaction_id: str) -> Transaction: ...
    def refund(self, transaction_id, amount_or_options=None) -> SuccessfulResult | ErrorResult | None: ...
    def sale(self, params) -> SuccessfulResult | ErrorResult | None: ...
    def search(self, *query) -> ResourceCollection: ...
    def submit_for_settlement(self, transaction_id, amount=None, params=None) -> SuccessfulResult | ErrorResult | None: ...
    def update_details(self, transaction_id, params=None) -> SuccessfulResult | ErrorResult | None: ...
    def submit_for_partial_settlement(self, transaction_id, amount, params=None) -> SuccessfulResult | ErrorResult | None: ...
    def package_tracking(self, transaction_id, params=None) -> SuccessfulResult | ErrorResult | None: ...
    def void(self, transaction_id, params=None) -> SuccessfulResult | ErrorResult | None: ...



from _typeshed import Incomplete
from collections.abc import Callable
from typing import type_check_only

from authlib.oauth2.rfc6750 import BearerTokenValidator
from authlib.oauth2.rfc7009 import RevocationEndpoint

@type_check_only
class _RevocationEndpoint(RevocationEndpoint):
    def query_token(self, token, token_type_hint): ...
    def revoke_token(self, token, request) -> None: ...

@type_check_only
class _BearerTokenValidator(BearerTokenValidator):
    def authenticate_token(self, token_string): ...

def create_query_client_func(session, client_model) -> Callable[[Incomplete], Incomplete]: ...
def create_save_token_func(session, token_model) -> Callable[[Incomplete, Incomplete], None]: ...
def create_query_token_func(session, token_model) -> Callable[[Incomplete, Incomplete], Incomplete]: ...
def create_revocation_endpoint(session, token_model) -> type[_RevocationEndpoint]: ...
def create_bearer_token_validator(session, token_model) -> type[_BearerTokenValidator]: ...



import ctypes
from _ctypes import _CData
from collections.abc import Callable, Iterable, Sequence
from ctypes import _SimpleCData, c_char
from multiprocessing.context import BaseContext
from multiprocessing.synchronize import _LockLike
from types import TracebackType
from typing import Any, Generic, Literal, Protocol, SupportsIndex, TypeVar, overload, type_check_only

__all__ = ["RawValue", "RawArray", "Value", "Array", "copy", "synchronized"]

_T = TypeVar("_T")
_CT = TypeVar("_CT", bound=_CData)

@overload
def RawValue(typecode_or_type: type[_CT], *args: Any) -> _CT: ...
@overload
def RawValue(typecode_or_type: str, *args: Any) -> Any: ...

@overload
def RawArray(typecode_or_type: type[_CT], size_or_initializer: int | Sequence[Any]) -> ctypes.Array[_CT]: ...
@overload
def RawArray(typecode_or_type: str, size_or_initializer: int | Sequence[Any]) -> Any: ...

@overload
def Value(typecode_or_type: type[_CT], *args: Any, lock: Literal[False], ctx: BaseContext | None = None) -> _CT: ...
@overload
def Value(
    typecode_or_type: type[_CT], *args: Any, lock: Literal[True] | _LockLike = True, ctx: BaseContext | None = None
) -> SynchronizedBase[_CT]: ...
@overload
def Value(
    typecode_or_type: str, *args: Any, lock: Literal[True] | _LockLike = True, ctx: BaseContext | None = None
) -> SynchronizedBase[Any]: ...
@overload
def Value(
    typecode_or_type: str | type[_CData], *args: Any, lock: bool | _LockLike = True, ctx: BaseContext | None = None
) -> Any: ...

@overload
def Array(
    typecode_or_type: type[_CT], size_or_initializer: int | Sequence[Any], *, lock: Literal[False], ctx: BaseContext | None = None
) -> _CT: ...
@overload
def Array(
    typecode_or_type: type[c_char],
    size_or_initializer: int | Sequence[Any],
    *,
    lock: Literal[True] | _LockLike = True,
    ctx: BaseContext | None = None,
) -> SynchronizedString: ...
@overload
def Array(
    typecode_or_type: type[_SimpleCData[_T]],
    size_or_initializer: int | Sequence[Any],
    *,
    lock: Literal[True] | _LockLike = True,
    ctx: BaseContext | None = None,
) -> SynchronizedArray[_T]: ...
@overload
def Array(
    typecode_or_type: str,
    size_or_initializer: int | Sequence[Any],
    *,
    lock: Literal[True] | _LockLike = True,
    ctx: BaseContext | None = None,
) -> SynchronizedArray[Any]: ...
@overload
def Array(
    typecode_or_type: str | type[_CData],
    size_or_initializer: int | Sequence[Any],
    *,
    lock: bool | _LockLike = True,
    ctx: BaseContext | None = None,
) -> Any: ...

def copy(obj: _CT) -> _CT: ...

@overload
def synchronized(obj: _SimpleCData[_T], lock: _LockLike | None = None, ctx: Any | None = None) -> Synchronized[_T]: ...
@overload
def synchronized(obj: ctypes.Array[c_char], lock: _LockLike | None = None, ctx: Any | None = None) -> SynchronizedString: ...
@overload
def synchronized(
    obj: ctypes.Array[_SimpleCData[_T]], lock: _LockLike | None = None, ctx: Any | None = None
) -> SynchronizedArray[_T]: ...
@overload
def synchronized(obj: _CT, lock: _LockLike | None = None, ctx: Any | None = None) -> SynchronizedBase[_CT]: ...

@type_check_only
class _AcquireFunc(Protocol):
    def __call__(self, block: bool = ..., timeout: float | None = ..., /) -> bool: ...

class SynchronizedBase(Generic[_CT]):
    acquire: _AcquireFunc
    release: Callable[[], None]
    def __init__(self, obj: Any, lock: _LockLike | None = None, ctx: Any | None = None) -> None: ...
    def __reduce__(self) -> tuple[Callable[[Any, _LockLike], SynchronizedBase[Any]], tuple[Any, _LockLike]]: ...
    def get_obj(self) -> _CT: ...
    def get_lock(self) -> _LockLike: ...
    def __enter__(self) -> bool: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None, /
    ) -> None: ...

class Synchronized(SynchronizedBase[_SimpleCData[_T]], Generic[_T]):
    value: _T

class SynchronizedArray(SynchronizedBase[ctypes.Array[_SimpleCData[_T]]], Generic[_T]):
    def __len__(self) -> int: ...

    @overload
    def __getitem__(self, i: slice[SupportsIndex | None]) -> list[_T]: ...
    @overload
    def __getitem__(self, i: SupportsIndex) -> _T: ...

    @overload
    def __setitem__(self, i: slice[SupportsIndex | None], value: Iterable[_T]) -> None: ...
    @overload
    def __setitem__(self, i: SupportsIndex, value: _T) -> None: ...

    def __getslice__(self, start: SupportsIndex, stop: SupportsIndex) -> list[_T]: ...
    def __setslice__(self, start: SupportsIndex, stop: SupportsIndex, values: Iterable[_T]) -> None: ...

class SynchronizedString(SynchronizedArray[bytes]):
    @overload  # type: ignore[override]
    def __getitem__(self, i: slice[SupportsIndex | None]) -> bytes: ...
    @overload
    def __getitem__(self, i: SupportsIndex) -> bytes: ...

    @overload  # type: ignore[override]
    def __setitem__(self, i: slice[SupportsIndex | None], value: bytes) -> None: ...
    @overload
    def __setitem__(self, i: SupportsIndex, value: bytes) -> None: ...

    def __getslice__(self, start: SupportsIndex, stop: SupportsIndex) -> bytes: ...  # type: ignore[override]
    def __setslice__(self, start: SupportsIndex, stop: SupportsIndex, values: bytes) -> None: ...  # type: ignore[override]

    value: bytes
    raw: bytes



from _typeshed import Incomplete
from typing_extensions import deprecated

@deprecated("Please use `JsonWebKey` directly.")
def loads(obj, kid=None): ...
@deprecated("Please use `JsonWebKey` directly.")
def dumps(key, kty=None, **params) -> dict[Incomplete, Incomplete]: ...



from typing import SupportsIndex

def comb(n: SupportsIndex, k: SupportsIndex, /) -> int: ...
def factorial(n: SupportsIndex, /) -> int: ...
def gcd(*integers: SupportsIndex) -> int: ...
def isqrt(n: SupportsIndex, /) -> int: ...
def lcm(*integers: SupportsIndex) -> int: ...
def perm(n: SupportsIndex, k: SupportsIndex | None = None, /) -> int: ...



from _typeshed import Incomplete, SupportsWrite
from collections.abc import Iterable
from typing import TypeVar

from pygments.formatter import Formatter
from pygments.token import _TokenType

_T = TypeVar("_T", str, bytes)

__all__ = ["HtmlFormatter"]

class HtmlFormatter(Formatter[_T]):
    title: Incomplete
    nowrap: Incomplete
    noclasses: Incomplete
    classprefix: Incomplete
    cssclass: Incomplete
    cssstyles: Incomplete
    prestyles: Incomplete
    cssfile: Incomplete
    noclobber_cssfile: Incomplete
    tagsfile: Incomplete
    tagurlformat: Incomplete
    filename: Incomplete
    wrapcode: Incomplete
    span_element_openers: Incomplete
    linenos: int
    linenostart: Incomplete
    linenostep: Incomplete
    linenospecial: Incomplete
    nobackground: Incomplete
    lineseparator: Incomplete
    lineanchors: Incomplete
    linespans: Incomplete
    anchorlinenos: Incomplete
    hl_lines: Incomplete
    def get_style_defs(self, arg=None): ...
    def get_token_style_defs(self, arg=None): ...
    def get_background_style_defs(self, arg=None): ...
    def get_linenos_style_defs(self): ...
    def get_css_prefix(self, arg): ...
    def wrap(self, source): ...
    def format_unencoded(self, tokensource: Iterable[tuple[_TokenType, str]], outfile: SupportsWrite[str]) -> None: ...



from typing import NoReturn

from authlib.oauth1 import ClientAuth
from authlib.oauth1.client import OAuth1Client

# Inherits from requests.auth.AuthBase
class OAuth1Auth(ClientAuth):
    def __call__(self, req): ...

# Inherits from requests.Session
class OAuth1Session(OAuth1Client):
    auth_class = OAuth1Auth
    def __init__(
        self,
        client_id,
        client_secret=None,
        token=None,
        token_secret=None,
        redirect_uri=None,
        rsa_key=None,
        verifier=None,
        signature_method=...,
        signature_type=...,
        force_include_body=False,
        **kwargs,
    ) -> None: ...
    def rebuild_auth(self, prepared_request, response) -> None: ...
    @staticmethod
    def handle_error(error_type: str | None, error_description: str | None) -> NoReturn: ...



from _typeshed import Incomplete, SupportsKeysAndGetItem
from collections.abc import Callable, Iterable, Mapping
from html.parser import HTMLParser
from logging import Logger
from typing import ClassVar, Final, Literal, TypeAlias

from fpdf import FPDF

from .enums import Align, TextEmphasis
from .fonts import FontFace
from .table import Row, Table

__author__: Final[str]
__copyright__: Final[str]

_OLType: TypeAlias = Literal["1", "a", "A", "I", "i"]

LOGGER: Logger
MESSAGE_WAITING_WIN1252: Final = "\x95"
BULLET_UNICODE: Final = "•"
DEGREE_SIGN_WIN1252: Final = "\xb0"
RING_OPERATOR_UNICODE: Final = "∘"
HEADING_TAGS: Final[tuple[str, ...]]
DEFAULT_TAG_STYLES: Final[dict[str, FontFace]]
INLINE_TAGS: Final[tuple[str, ...]]
BLOCK_TAGS: Final[tuple[str, ...]]

COLOR_DICT: Final[dict[str, str]]

def color_as_decimal(color: str | None = "#000000") -> tuple[int, int, int] | None: ...
def parse_css_style(style_attr: str) -> dict[str, str]: ...

class HTML2FPDF(HTMLParser):
    HTML_UNCLOSED_TAGS: ClassVar[tuple[str, ...]]
    TABLE_LINE_HEIGHT: ClassVar[float]

    pdf: FPDF
    image_map: Callable[[str], str]
    ul_bullet_char: str
    li_prefix_color: tuple[int, int, int]
    warn_on_tags_not_matching: bool

    font_family: str
    font_size_pt: float
    font_emphasis: TextEmphasis
    font_color: tuple[int, int, int]

    style_stack: list[FontFace]
    h: float
    follows_trailing_space: bool
    follows_heading: bool
    href: str
    align: float | Align | None
    indent: int
    line_height_stack: list[Incomplete]
    ol_type: dict[int, _OLType]
    bullet: list[Incomplete]
    heading_level: Incomplete | None
    render_title_tag: bool
    table_line_separators: bool
    table: Table | None
    table_row: Row | None
    tr: dict[str, str] | None
    td_th: dict[str, str] | None
    tag_indents: dict[str, int]
    tag_styles: dict[str, FontFace]

    def __init__(
        self,
        pdf: FPDF,
        image_map: Callable[[str], str] | None = None,
        li_tag_indent: int | None = None,
        dd_tag_indent: int | None = None,
        table_line_separators: bool = False,
        ul_bullet_char: str = "disc",
        li_prefix_color: tuple[int, int, int] = (190, 0, 0),
        heading_sizes: SupportsKeysAndGetItem[str, int] | Iterable[tuple[str, int]] | None = None,
        pre_code_font: str | None = None,
        warn_on_tags_not_matching: bool = True,
        tag_indents: dict[str, int] | None = None,
        tag_styles: Mapping[str, FontFace] | None = None,
        font_family: str = "times",
        render_title_tag: bool = False,
    ) -> None: ...
    def handle_data(self, data) -> None: ...
    def handle_starttag(self, tag, attrs) -> None: ...
    def handle_endtag(self, tag) -> None: ...
    def put_link(self, text) -> None: ...
    def render_toc(self, pdf, outline) -> None: ...
    def error(self, message: str) -> None: ...

def ul_prefix(ul_type: str, is_ttf_font: bool | None) -> str: ...
def ol_prefix(ol_type: _OLType, index: int) -> str: ...

class HTMLMixin:
    def __init__(self, *args, **kwargs) -> None: ...



import asyncio
import socket
from _typeshed import Incomplete
from typing import ClassVar, Final

MAGIC: Final = b"GD"
VERSION: Final = 0x01
MSG_TYPE_REQUEST: Final = 0x01
MSG_TYPE_RESPONSE: Final = 0x02
MSG_TYPE_ERROR: Final = 0x03
MSG_TYPE_CHUNK: Final = 0x04
MSG_TYPE_END: Final = 0x05
MSG_TYPE_STASH: Final = 0x10
MSG_TYPE_STATUS: Final = 0x11
MSG_TYPE_MANAGE: Final = 0x12
MSG_TYPE_REQUEST_STR: Final = "request"
MSG_TYPE_RESPONSE_STR: Final = "response"
MSG_TYPE_ERROR_STR: Final = "error"
MSG_TYPE_CHUNK_STR: Final = "chunk"
MSG_TYPE_END_STR: Final = "end"
MSG_TYPE_STASH_STR: Final = "stash"
MSG_TYPE_STATUS_STR: Final = "status"
MSG_TYPE_MANAGE_STR: Final = "manage"
MSG_TYPE_TO_STR: Final[dict[int, str]]
MSG_TYPE_FROM_STR: Final[dict[str, int]]
STASH_OP_PUT: Final = 1
STASH_OP_GET: Final = 2
STASH_OP_DELETE: Final = 3
STASH_OP_KEYS: Final = 4
STASH_OP_CLEAR: Final = 5
STASH_OP_INFO: Final = 6
STASH_OP_ENSURE: Final = 7
STASH_OP_DELETE_TABLE: Final = 8
STASH_OP_TABLES: Final = 9
STASH_OP_EXISTS: Final = 10
MANAGE_OP_ADD: Final = 1
MANAGE_OP_REMOVE: Final = 2
HEADER_FORMAT: Final = ">2sBBIQ"
HEADER_SIZE: Final[int]
MAX_MESSAGE_SIZE: Final = 67108864

class BinaryProtocol:
    HEADER_SIZE: ClassVar[int]
    MAX_MESSAGE_SIZE: ClassVar[int]
    MSG_TYPE_REQUEST: ClassVar[str]
    MSG_TYPE_RESPONSE: ClassVar[str]
    MSG_TYPE_ERROR: ClassVar[str]
    MSG_TYPE_CHUNK: ClassVar[str]
    MSG_TYPE_END: ClassVar[str]
    MSG_TYPE_STASH: ClassVar[str]
    MSG_TYPE_STATUS: ClassVar[str]
    MSG_TYPE_MANAGE: ClassVar[str]

    @staticmethod
    def encode_header(msg_type: int, request_id: int, payload_length: int) -> bytes: ...
    @staticmethod
    def decode_header(data: bytes) -> tuple[int, int, int]: ...
    @staticmethod
    def encode_request(
        request_id: int,
        app_path: str,
        action: str,
        args: tuple[Incomplete, ...] | None = None,
        kwargs: dict[str, Incomplete] | None = None,
    ) -> bytes: ...
    @staticmethod
    def encode_response(request_id: int, result) -> bytes: ...
    @staticmethod
    def encode_error(request_id: int, error: BaseException | dict[str, Incomplete]) -> bytes: ...
    @staticmethod
    def encode_chunk(request_id: int, data) -> bytes: ...
    @staticmethod
    def encode_end(request_id: int) -> bytes: ...
    @staticmethod
    def encode_status(request_id: int) -> bytes: ...
    @staticmethod
    def encode_manage(request_id: int, op: int, count: int = 1) -> bytes: ...
    @staticmethod
    def encode_stash(request_id: int, op: int, table: str, key=None, value=None, pattern=None) -> bytes: ...
    @staticmethod
    def decode_message(data: bytes) -> tuple[str, int, Incomplete]: ...
    @staticmethod
    async def read_message_async(reader: asyncio.StreamReader) -> dict[str, Incomplete]: ...
    @staticmethod
    async def write_message_async(writer: asyncio.StreamWriter, message: dict[str, Incomplete]) -> None: ...
    @staticmethod
    def _recv_exactly(sock: socket.socket, n: int) -> bytes: ...
    @staticmethod
    def read_message(sock: socket.socket) -> dict[str, Incomplete]: ...
    @staticmethod
    def write_message(sock: socket.socket, message: dict[str, Incomplete]) -> None: ...
    @staticmethod
    def _encode_from_dict(message: dict[str, Incomplete]) -> bytes: ...

DirtyProtocol = BinaryProtocol

# TODO: Use TypedDict for results
def make_request(
    request_id: int | str,
    app_path: str,
    action: str,
    args: tuple[Incomplete, ...] | None = None,
    kwargs: dict[str, Incomplete] | None = None,
) -> dict[str, Incomplete]: ...
def make_response(request_id: int | str, result) -> dict[str, Incomplete]: ...
def make_error_response(request_id: int | str, error) -> dict[str, Incomplete]: ...
def make_chunk_message(request_id: int | str, data) -> dict[str, Incomplete]: ...
def make_end_message(request_id: int | str) -> dict[str, Incomplete]: ...
def make_stash_message(
    request_id: int | str, op: int, table: str, key=None, value=None, pattern=None
) -> dict[str, Incomplete]: ...
def make_manage_message(request_id: int | str, op: int, count: int = 1) -> dict[str, Incomplete]: ...



from _typeshed import Incomplete
from types import TracebackType
from typing import Any, ClassVar
from typing_extensions import Self

class DirtyClient:
    socket_path: str
    timeout: float

    def __init__(self, socket_path: str, timeout: float = 30.0) -> None: ...
    def connect(self) -> None: ...
    # Arguments and result depend on app path and method name passed to action
    def execute(self, app_path: str, action: str, *args: Any, **kwargs: Any) -> Any: ...
    def stream(self, app_path: str, action: str, *args: Any, **kwargs: Any) -> DirtyStreamIterator: ...
    def close(self) -> None: ...
    async def connect_async(self) -> None: ...
    async def execute_async(self, app_path: str, action: str, *args: Any, **kwargs: Any) -> Any: ...
    def stream_async(self, app_path: str, action: str, *args: Any, **kwargs: Any) -> DirtyAsyncStreamIterator: ...
    async def close_async(self) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> None: ...
    async def __aenter__(self) -> Self: ...
    async def __aexit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> None: ...

class DirtyStreamIterator:
    DEFAULT_IDLE_TIMEOUT: ClassVar[float]
    client: DirtyClient
    app_path: str
    action: str
    args: tuple[Incomplete, ...]
    kwargs: dict[str, Incomplete]

    def __init__(
        self,
        client: DirtyClient,
        app_path: str,
        action: str,
        args: tuple[Incomplete, ...],
        kwargs: dict[str, Incomplete],
        idle_timeout: float | None = None,
    ) -> None: ...
    def __iter__(self) -> Self: ...
    def __next__(self) -> Incomplete | None: ...

class DirtyAsyncStreamIterator:
    DEFAULT_IDLE_TIMEOUT: ClassVar[float]
    client: DirtyClient
    app_path: str
    action: str
    args: tuple[Incomplete, ...]
    kwargs: dict[str, Incomplete]

    def __init__(
        self,
        client: DirtyClient,
        app_path: str,
        action: str,
        args: tuple[Incomplete, ...],
        kwargs: dict[str, Incomplete],
        idle_timeout: float | None = None,
    ) -> None: ...
    def __aiter__(self) -> Self: ...
    async def __anext__(self) -> Incomplete | None: ...

def set_dirty_socket_path(path: str) -> None: ...
def get_dirty_socket_path() -> str: ...
def get_dirty_client(timeout: float = 30.0) -> DirtyClient: ...
async def get_dirty_client_async(timeout: float = 30.0) -> DirtyClient: ...
def close_dirty_client() -> None: ...
async def close_dirty_client_async() -> None: ...



from typing import Any

from google._upb._message import MessageMeta

MESSAGE_CLASS_CACHE: dict[str, Any]

class GeneratedProtocolMessageType(MessageMeta):
    def __new__(  # noqa: Y034
        cls, name: str, bases: tuple[type, ...], dictionary: dict[str, Any]
    ) -> GeneratedProtocolMessageType: ...



import re
from _typeshed import Incomplete, ReadableBuffer
from collections.abc import Sequence
from typing import Final

__version__: Final[str]
CANNOT_START_LINE: Final[Sequence[str]]
ALL_CANNOT_START: Final[str]
CANNOT_END_LINE: Final[Sequence[str]]
ALL_CANNOT_END: Final[str]

def is_multi_byte(ch: str | bytes | bytearray) -> bool: ...
def getCharWidths(word: str, fontName: str, fontSize: float) -> list[float]: ...
def wordSplit(word, maxWidths, fontName, fontSize, encoding: str = "utf8") -> list[list[Incomplete]]: ...
def dumbSplit(word, widths, maxWidths) -> list[list[Incomplete]]: ...
def kinsokuShoriSplit(word, widths, availWidth) -> None: ...

rx: re.Pattern[str]

def cjkwrap(text: ReadableBuffer, width: float, encoding: str = "utf8"): ...



from _typeshed import Incomplete, ReadableBuffer
from collections.abc import Iterable
from typing import ClassVar, Final, SupportsBytes, SupportsIndex

from authlib.jose.rfc7516 import JWEAlgorithm

from .ec_key import ECKey
from .oct_key import OctKey
from .rsa_key import RSAKey

class DirectAlgorithm(JWEAlgorithm):
    name: str
    description: str
    def prepare_key(self, raw_data) -> OctKey: ...
    def generate_preset(self, enc_alg, key) -> dict[Incomplete, Incomplete]: ...
    def wrap(self, enc_alg, headers, key, preset=None) -> dict[str, Incomplete]: ...
    def unwrap(self, enc_alg, ek, headers, key): ...

class RSAAlgorithm(JWEAlgorithm):
    key_size: int
    name: str
    deprecated: bool
    description: str
    padding: Incomplete
    def __init__(self, name: str, description: str, pad_fn) -> None: ...
    def prepare_key(self, raw_data) -> RSAKey: ...
    def generate_preset(self, enc_alg, key) -> dict[str, Incomplete]: ...
    def wrap(self, enc_alg, headers, key, preset=None) -> dict[str, Incomplete]: ...
    def unwrap(self, enc_alg, ek, headers, key): ...

class AESAlgorithm(JWEAlgorithm):
    name: str
    description: str
    key_size: int
    def __init__(self, key_size: int) -> None: ...
    def prepare_key(self, raw_data) -> OctKey: ...
    def generate_preset(self, enc_alg, key) -> dict[str, Incomplete]: ...
    def wrap_cek(self, cek, key) -> dict[str, Incomplete]: ...
    def wrap(self, enc_alg, headers, key, preset=None) -> dict[str, Incomplete]: ...
    def unwrap(self, enc_alg, ek, headers, key) -> bytes: ...

class AESGCMAlgorithm(JWEAlgorithm):
    EXTRA_HEADERS: ClassVar[Iterable[str]]
    name: str
    description: str
    key_size: int
    def __init__(self, key_size: int) -> None: ...
    def prepare_key(self, raw_data) -> OctKey: ...
    def generate_preset(self, enc_alg, key) -> dict[str, Incomplete]: ...
    def wrap(self, enc_alg, headers, key, preset=None) -> dict[str, Incomplete]: ...
    def unwrap(self, enc_alg, ek, headers, key) -> bytes: ...

class ECDHESAlgorithm(JWEAlgorithm):
    EXTRA_HEADERS: ClassVar[Iterable[str]]
    ALLOWED_KEY_CLS = Incomplete
    name: str
    description: str
    key_size: int | None
    aeskw: AESAlgorithm
    def __init__(self, key_size: int | None = None) -> None: ...
    def prepare_key(self, raw_data) -> ECKey: ...
    def generate_preset(self, enc_alg, key) -> dict[str, Incomplete]: ...
    def compute_fixed_info(self, headers, bit_size) -> bytes: ...
    def compute_derived_key(self, shared_key, fixed_info, bit_size) -> bytes: ...
    def deliver(self, key, pubkey, headers, bit_size) -> bytes: ...
    def wrap(self, enc_alg, headers, key, preset=None) -> dict[str, Incomplete]: ...
    def unwrap(self, enc_alg, ek, headers, key) -> bytes: ...

def u32be_len_input(
    s: str | bytes | float | Iterable[SupportsIndex] | SupportsIndex | SupportsBytes | ReadableBuffer, base64: bool = False
) -> bytes: ...

JWE_ALG_ALGORITHMS: Final[list[JWEAlgorithm]]



from typing import Final

from .bytes import parse_stylesheet_bytes as parse_stylesheet_bytes
from .parser import (
    parse_blocks_contents as parse_blocks_contents,
    parse_declaration_list as parse_declaration_list,
    parse_one_component_value as parse_one_component_value,
    parse_one_declaration as parse_one_declaration,
    parse_one_rule as parse_one_rule,
    parse_rule_list as parse_rule_list,
    parse_stylesheet as parse_stylesheet,
)
from .serializer import serialize as serialize, serialize_identifier as serialize_identifier
from .tokenizer import parse_component_value_list as parse_component_value_list

__version__: Final[str]
VERSION: Final[str]



import datetime
from typing import Any, Final, Literal, TypedDict, type_check_only

from dateparser.conf import Settings

from .date import DateDataParser, _DetectLanguagesFunction

__version__: Final[str]

_default_parser: DateDataParser

@type_check_only
class _Settings(TypedDict, total=False):  # noqa: Y049
    DATE_ORDER: str
    PREFER_LOCALE_DATE_ORDER: bool
    TIMEZONE: str
    TO_TIMEZONE: str
    RETURN_AS_TIMEZONE_AWARE: bool
    PREFER_MONTH_OF_YEAR: Literal["current", "first", "last"]
    PREFER_DAY_OF_MONTH: Literal["current", "first", "last"]
    PREFER_DATES_FROM: Literal["current_period", "future", "past"]
    RELATIVE_BASE: datetime.datetime
    STRICT_PARSING: bool
    REQUIRE_PARTS: list[Literal["day", "month", "year"]]
    SKIP_TOKENS: list[str]
    NORMALIZE: bool
    RETURN_TIME_AS_PERIOD: bool
    RETURN_TIME_SPAN: bool
    DEFAULT_START_OF_WEEK: Literal["monday", "sunday"]
    DEFAULT_DAYS_IN_MONTH: int
    PARSERS: list[Literal["timestamp", "relative-time", "custom-formats", "absolute-time", "no-spaces-time"]]
    DEFAULT_LANGUAGES: list[str]
    LANGUAGE_DETECTION_CONFIDENCE_THRESHOLD: float
    CACHE_SIZE_LIMIT: int

def parse(
    date_string: str,
    date_formats: list[str] | tuple[str, ...] | set[str] | None = None,
    languages: list[str] | tuple[str, ...] | set[str] | None = None,
    locales: list[str] | tuple[str, ...] | set[str] | None = None,
    region: str | None = None,
    settings: Settings | dict[str, Any] | None = None,
    detect_languages_function: _DetectLanguagesFunction | None = None,
) -> datetime.datetime | None: ...



from _typeshed import FileDescriptorOrPath, Unused
from collections.abc import Iterable
from typing import Final, type_check_only

RequirePyRXP: int
simpleparse: int

@type_check_only
class _smartDecode:
    @staticmethod
    def __call__(s: str | bytes | bytearray) -> str: ...

smartDecode: _smartDecode
NONAME: Final = ""
NAMEKEY: Final = 0
CONTENTSKEY: Final = 1
CDATAMARKER: Final = "<![CDATA["
LENCDATAMARKER: Final = 9
CDATAENDMARKER: Final = "]]>"
replacelist: list[tuple[str, str]]

def unEscapeContentList(contentList: Iterable[str]) -> list[str]: ...
def parsexmlSimple(xmltext, oneOutermostTag: int = 0, eoCB: Unused = None, entityReplacer=...): ...

parsexml = parsexmlSimple

def parseFile(filename: FileDescriptorOrPath): ...

verbose: int

def skip_prologue(text, cursor): ...
def parsexml0(xmltext, startingat: int = 0, toplevel: int = 1, entityReplacer=...): ...
def pprettyprint(parsedxml): ...
def testparse(s, dump: int = 0) -> None: ...
def test(dump: int = 0) -> None: ...



from _typeshed import Unused
from typing import Any
from typing_extensions import Self

from .descriptor import (
    Descriptor,
    EnumDescriptor,
    FieldDescriptor,
    FileDescriptor,
    MethodDescriptor,
    OneofDescriptor,
    ServiceDescriptor,
)
from .descriptor_pb2 import FeatureSetDefaults, FileDescriptorProto

class DescriptorPool:
    def __new__(cls, descriptor_db: Any = None) -> Self: ...
    def __init__(  # pyright: ignore[reportInconsistentConstructor]
        self, descriptor_db: Any = None, use_deprecated_legacy_json_field_conflicts: Unused = False
    ) -> None: ...
    def Add(self, file_desc_proto: FileDescriptorProto) -> None: ...
    def AddSerializedFile(self, serialized_file_desc_proto: bytes) -> FileDescriptor: ...
    def FindFileByName(self, file_name: str) -> FileDescriptor: ...
    def FindFileContainingSymbol(self, symbol: str) -> FileDescriptor: ...
    def FindMessageTypeByName(self, full_name: str) -> Descriptor: ...
    def FindEnumTypeByName(self, full_name: str) -> EnumDescriptor: ...
    def FindFieldByName(self, full_name: str) -> FieldDescriptor: ...
    def FindOneofByName(self, full_name: str) -> OneofDescriptor: ...
    def FindExtensionByName(self, full_name: str) -> FieldDescriptor: ...
    def FindExtensionByNumber(self, message_descriptor: Descriptor, number: int) -> FieldDescriptor: ...
    def FindAllExtensions(self, message_descriptor: Descriptor) -> list[FieldDescriptor]: ...
    def FindServiceByName(self, full_name: str) -> ServiceDescriptor: ...
    def FindMethodByName(self, full_name: str) -> MethodDescriptor: ...
    def SetFeatureSetDefaults(self, defaults: FeatureSetDefaults) -> None: ...

def Default() -> DescriptorPool: ...



from _typeshed import Incomplete

from ..base_client import BaseOAuth, OAuthError as OAuthError
from .apps import FlaskOAuth1App as FlaskOAuth1App, FlaskOAuth2App as FlaskOAuth2App
from .integration import FlaskIntegration as FlaskIntegration, token_update as token_update

class OAuth(BaseOAuth):
    oauth1_client_cls = FlaskOAuth1App
    oauth2_client_cls = FlaskOAuth2App
    framework_integration_cls = FlaskIntegration
    app: Incomplete
    def __init__(self, app=None, cache=None, fetch_token=None, update_token=None): ...
    cache: Incomplete
    fetch_token: Incomplete
    update_token: Incomplete
    def init_app(self, app, cache=None, fetch_token=None, update_token=None): ...
    def create_client(self, name): ...
    def register(self, name, overwrite=False, **kwargs): ...

__all__ = ["OAuth", "FlaskIntegration", "FlaskOAuth1App", "FlaskOAuth2App", "token_update", "OAuthError"]



from _typeshed import Incomplete
from typing import Literal, TypedDict, type_check_only
from typing_extensions import NotRequired

from gunicorn.arbiter import Arbiter

@type_check_only
class _Worker(TypedDict):
    pid: int
    age: int
    booted: bool
    last_heartbeat: float
    aborted: NotRequired[bool]
    apps: NotRequired[list[str]]

@type_check_only
class _ShowWorkersReturnType(TypedDict):
    workers: list[_Worker]
    count: int

@type_check_only
class _App(TypedDict):
    import_path: str
    worker_count: int | None
    current_workers: int
    worker_pids: list[int]

@type_check_only
class _ShowDirtyReturnType(TypedDict):
    enabled: bool
    pid: int | None
    workers: list[_Worker]
    apps: list[_App]

@type_check_only
class _ShowStatsReturnType(TypedDict):
    uptime: float | None
    pid: int
    workers_current: int
    workers_target: int
    workers_spawned: int
    workers_killed: int
    reloads: int
    dirty_arbiter_pid: int | None

@type_check_only
class _ListenerInfo(TypedDict):
    address: str
    fd: int
    type: Literal["unix", "tcp", "tcp6", "unknown"]

@type_check_only
class _ShowListenersReturnType(TypedDict):
    listeners: list[_ListenerInfo]
    count: int

@type_check_only
class _WorkerAddReturnType(TypedDict):
    added: int
    previous: int
    total: int

@type_check_only
class _WorkerRemovedReturnType(TypedDict):
    removed: int
    previous: int
    total: int

@type_check_only
class _WorkerKillSucessReturnType(TypedDict):
    success: Literal[True]
    killed: int

@type_check_only
class _WorkerKillFailedReturnType(TypedDict):
    success: Literal[False]
    error: str

@type_check_only
class _ReloadReturnType(TypedDict):
    status: Literal["reloading"]

@type_check_only
class _ReopenReturnType(TypedDict):
    status: Literal["reopening"]

@type_check_only
class _ShutdownReturnType(TypedDict):
    status: Literal["shutting_down"]
    mode: str

@type_check_only
class _HelpReturnType(TypedDict):
    commands: dict[str, str]

class CommandHandlers:
    arbiter: Arbiter
    def __init__(self, arbiter: Arbiter) -> None: ...
    def show_workers(self) -> _ShowWorkersReturnType: ...
    def show_dirty(self) -> _ShowDirtyReturnType: ...
    def show_config(self) -> dict[str, Incomplete]: ...
    def show_stats(self) -> _ShowStatsReturnType: ...
    def show_listeners(self) -> _ShowListenersReturnType: ...
    def worker_add(self, count: int = 1) -> _WorkerAddReturnType: ...
    def worker_remove(self, count: int = 1) -> _WorkerRemovedReturnType: ...
    def worker_kill(self, pid: int) -> _WorkerKillSucessReturnType | _WorkerKillFailedReturnType: ...
    def dirty_add(self, count: int = 1) -> dict[str, Incomplete]: ...
    def dirty_remove(self, count: int = 1) -> dict[str, Incomplete]: ...
    def reload(self) -> _ReloadReturnType: ...
    def reopen(self) -> _ReopenReturnType: ...
    def shutdown(self, mode: str = "graceful") -> _ShutdownReturnType: ...
    def show_all(self) -> dict[str, Incomplete]: ...
    def help(self) -> _HelpReturnType: ...



from authlib.oauth2.rfc6749 import JsonPayload, JsonRequest, OAuth2Payload, OAuth2Request

class DjangoOAuth2Payload(OAuth2Payload):
    def __init__(self, request) -> None: ...

class DjangoOAuth2Request(OAuth2Request):
    payload: DjangoOAuth2Payload
    def __init__(self, request) -> None: ...
    @property
    def args(self): ...
    @property
    def form(self): ...

class DjangoJsonPayload(JsonPayload):
    def __init__(self, request) -> None: ...
    def data(self): ...

class DjangoJsonRequest(JsonRequest):
    payload: DjangoJsonPayload
    def __init__(self, request) -> None: ...



from datetime import datetime, tzinfo

from google.protobuf.timestamp_pb2 import Timestamp

def from_json_string(value: str) -> Timestamp: ...
def from_microseconds(micros: float) -> Timestamp: ...
def from_milliseconds(millis: float) -> Timestamp: ...
def from_nanoseconds(nanos: float) -> Timestamp: ...
def from_seconds(seconds: float) -> Timestamp: ...
def from_current_time() -> Timestamp: ...
def to_json_string(ts: Timestamp) -> str: ...
def to_microseconds(ts: Timestamp) -> int: ...
def to_milliseconds(ts: Timestamp) -> int: ...
def to_nanoseconds(ts: Timestamp) -> int: ...
def to_seconds(ts: Timestamp) -> int: ...
def to_datetime(ts: Timestamp, tz: tzinfo | None = None) -> datetime: ...



import sys

if sys.platform == "linux":
    from collections.abc import Sequence
    from typing import Final

    RLIMIT_AS: Final[int]
    RLIMIT_CORE: Final[int]
    RLIMIT_CPU: Final[int]
    RLIMIT_DATA: Final[int]
    RLIMIT_FSIZE: Final[int]
    RLIMIT_MEMLOCK: Final[int]
    RLIMIT_NOFILE: Final[int]
    RLIMIT_NPROC: Final[int]
    RLIMIT_RSS: Final[int]
    RLIMIT_STACK: Final[int]
    RLIMIT_LOCKS: Final[int]
    RLIMIT_MSGQUEUE: Final[int]
    RLIMIT_NICE: Final[int]
    RLIMIT_RTPRIO: Final[int]
    RLIMIT_RTTIME: Final[int]
    RLIMIT_SIGPENDING: Final[int]
    RLIM_INFINITY: Final[int]

    def getpagesize() -> int: ...
    def net_if_addrs() -> list[tuple[str, int, str, str | None, str | None, str | None]]: ...
    def net_if_flags(nic_name: str, /) -> list[str]: ...
    def net_if_is_running(nic_name: str, /) -> bool: ...
    def net_if_mtu(nic_name: str, /) -> int: ...
    def proc_priority_get(pid: int, /) -> int: ...
    def proc_priority_set(pid: int, priority: int, /) -> None: ...
    def users() -> list[tuple[str, str, str, float, int]]: ...

    version: Final[int]
    DUPLEX_FULL: Final[int]
    DUPLEX_HALF: Final[int]
    DUPLEX_UNKNOWN: Final[int]

    def proc_ioprio_get(pid: int, /) -> tuple[int, int]: ...
    def proc_ioprio_set(pid: int, ioclass: int, iodata: int, /) -> None: ...
    def proc_cpu_affinity_get(pid: int, /) -> list[int]: ...
    def proc_cpu_affinity_set(pid: int, cpu_set: Sequence[int], /) -> None: ...
    def disk_partitions(mtab_path: str, /) -> list[tuple[str, str, str, str]]: ...
    def net_if_duplex_speed(nic_name: str, /) -> tuple[int, int]: ...  # It's actually list of 2 elements
    def heap_info() -> tuple[int, int]: ...
    def heap_trim() -> bool: ...
    def linux_sysinfo() -> tuple[int, int, int, int, int, int, int]: ...
    def check_pid_range(pid: int, /) -> None: ...
    def set_debug(value: bool, /) -> None: ...



from _typeshed import Incomplete
from collections.abc import Iterator
from types import TracebackType
from typing_extensions import Self

from .errors import DirtyError

class StashError(DirtyError): ...

class StashTableNotFoundError(StashError):
    table_name: str

    def __init__(self, table_name: str) -> None: ...

class StashKeyNotFoundError(StashError):
    table_name: str
    key: str

    def __init__(self, table_name: str, key: str) -> None: ...

class StashClient:
    socket_path: str
    timeout: float

    def __init__(self, socket_path: str, timeout: float = 30.0) -> None: ...
    def put(self, table: str, key: str, value) -> None: ...
    def get(self, table: str, key: str, default=None): ...
    def delete(self, table: str, key: str) -> bool: ...
    def keys(self, table: str, pattern: str | None = None) -> list[str]: ...
    def clear(self, table: str) -> None: ...
    def info(self, table: str) -> dict[str, Incomplete]: ...
    def ensure(self, table: str) -> None: ...
    def exists(self, table: str, key=None) -> bool: ...
    def delete_table(self, table: str) -> None: ...
    def tables(self) -> list[str]: ...
    def table(self, name: str) -> StashTable: ...
    def close(self) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> None: ...

class StashTable:
    def __init__(self, client: StashClient, name: str) -> None: ...
    @property
    def name(self) -> str: ...
    def __getitem__(self, key: str): ...
    def __setitem__(self, key: str, value) -> None: ...
    def __delitem__(self, key: str) -> None: ...
    def __contains__(self, key: str) -> bool: ...
    def __iter__(self) -> Iterator[str]: ...
    def __len__(self) -> int: ...
    def get(self, key: str, default=None): ...
    def keys(self, pattern: str | None = None) -> list[str]: ...
    def clear(self) -> None: ...
    def items(self) -> Iterator[tuple[Incomplete, Incomplete]]: ...
    def values(self) -> Iterator[Incomplete]: ...

def set_stash_socket_path(path: str) -> None: ...
def get_stash_socket_path() -> str: ...
def put(table: str, key: str, value) -> None: ...
def get(table: str, key: str, default=None): ...
def delete(table: str, key: str) -> bool: ...
def keys(table: str, pattern: str | None = None) -> list[str]: ...
def clear(table: str) -> None: ...
def info(table: str) -> dict[str, Incomplete]: ...
def ensure(table: str) -> None: ...
def exists(table: str, key: str | None = None) -> bool: ...
def delete_table(table: str) -> None: ...
def tables() -> list[str]: ...
def table(name: str) -> StashTable: ...



from _typeshed import Incomplete
from logging import Logger
from typing import ClassVar

class LanguageValidator:
    VALID_KEYS: ClassVar[list[str]]
    logger: Logger | None
    @classmethod
    def get_logger(cls) -> Logger: ...
    @classmethod
    def validate_info(cls, language_id, info: dict[str, Incomplete]) -> bool: ...



from io import BytesIO
from typing import TypeVar

from google.protobuf.message import Message

_MessageT = TypeVar("_MessageT", bound=Message)

def serialize(message: Message, deterministic: bool | None = None) -> bytes: ...
def parse(message_class: type[_MessageT], payload: bytes) -> _MessageT: ...
def serialize_length_prefixed(message: Message, output: BytesIO) -> None: ...
def parse_length_prefixed(message_class: type[_MessageT], input_bytes: BytesIO) -> _MessageT: ...
def byte_size(message: Message) -> int: ...
def clear_message(message: Message) -> None: ...
def clear_field(message: Message, field_name: str) -> None: ...



from _typeshed import StrOrBytesPath
from abc import ABC, abstractmethod
from collections.abc import Sequence
from typing import TypeAlias

from _remote_debugging import AwaitedInfo, FrameInfo, InterpreterInfo, LocationInfo

_Location: TypeAlias = int | tuple[int, int, int, int] | LocationInfo | None
_Frame: TypeAlias = FrameInfo | tuple[str, _Location, str, int | None]
_Timestamps: TypeAlias = Sequence[int] | None

def normalize_location(location: _Location) -> tuple[int, int, int, int]: ...
def extract_lineno(location: _Location) -> int: ...
def filter_internal_frames(frames: Sequence[_Frame]) -> list[_Frame]: ...
def iter_async_frames(awaited_info_list: Sequence[AwaitedInfo]) -> object: ...

class Collector(ABC):
    @abstractmethod
    def collect(
        self, stack_frames: Sequence[InterpreterInfo] | Sequence[AwaitedInfo], timestamps_us: _Timestamps = None
    ) -> None: ...
    def collect_failed_sample(self) -> None: ...
    @abstractmethod
    def export(self, filename: StrOrBytesPath) -> None: ...



from _typeshed import Incomplete
from argparse import Namespace
from ast import AST
from collections.abc import Generator
from logging import Logger
from tokenize import TokenInfo
from typing import Any, Final, TypeAlias

from .plugins.finder import LoadedPlugin

LOG: Logger
NEWLINE: Final[frozenset[int]]
SKIP_TOKENS: Final[frozenset[int]]

_LogicalMapping: TypeAlias = list[tuple[int, tuple[int, int]]]
_Logical: TypeAlias = tuple[list[str], list[str], _LogicalMapping]

class FileProcessor:
    noqa: bool
    options: Incomplete
    filename: Incomplete
    lines: Incomplete
    blank_before: int
    blank_lines: int
    checker_state: Incomplete
    hang_closing: Incomplete
    indent_char: Incomplete
    indent_level: int
    indent_size: Incomplete
    line_number: int
    logical_line: str
    max_line_length: Incomplete
    max_doc_length: Incomplete
    multiline: bool
    previous_indent_level: int
    previous_logical: str
    previous_unindented_logical_line: str
    tokens: Incomplete
    total_lines: Incomplete
    verbose: Incomplete
    statistics: Incomplete
    def __init__(self, filename: str, options: Namespace, lines: list[str] | None = None) -> None: ...
    @property
    def file_tokens(self) -> list[TokenInfo]: ...
    def tstring_start(self, lineno: int) -> None: ...
    def fstring_start(self, lineno: int) -> None: ...
    def multiline_string(self, token: TokenInfo) -> Generator[str]: ...
    def reset_blank_before(self) -> None: ...
    def delete_first_token(self) -> None: ...
    def visited_new_blank_line(self) -> None: ...
    def update_state(self, mapping: _LogicalMapping) -> None: ...
    def update_checker_state_for(self, plugin: LoadedPlugin) -> None: ...
    def next_logical_line(self) -> None: ...
    def build_logical_line_tokens(self) -> _Logical: ...
    def build_ast(self) -> AST: ...
    def build_logical_line(self) -> tuple[str, str, _LogicalMapping]: ...
    def keyword_arguments_for(self, parameters: dict[str, bool], arguments: dict[str, Any]) -> dict[str, Any]: ...
    def generate_tokens(self) -> Generator[TokenInfo]: ...
    def noqa_line_for(self, line_number: int) -> str | None: ...
    def next_line(self) -> str: ...
    def read_lines(self) -> list[str]: ...
    def read_lines_from_filename(self) -> list[str]: ...
    def read_lines_from_stdin(self) -> list[str]: ...
    def should_ignore_file(self) -> bool: ...
    def strip_utf_bom(self) -> None: ...

def is_eol_token(token: TokenInfo) -> bool: ...
def is_multiline_string(token: TokenInfo) -> bool: ...
def token_is_newline(token: TokenInfo) -> bool: ...
def count_parentheses(current_parentheses_count: int, token_text: str) -> int: ...
def expand_indent(line: str) -> int: ...
def mutate_string(text: str) -> str: ...



from _typeshed import Incomplete
from asyncio import StreamReader, StreamWriter
from collections.abc import Iterable, Mapping
from signal import Signals
from typing import Any, ClassVar, Literal

from gunicorn.config import Config
from gunicorn.glogging import Logger as GLogger
from gunicorn.workers.workertmp import WorkerTmp

class DirtyWorker:
    SIGNALS: ClassVar[list[Signals]]
    age: int
    pid: int | Literal["[booting]"]
    ppid: int
    app_paths: list[str]
    cfg: Config
    log: GLogger
    socket_path: str
    booted: bool
    aborted: bool
    alive: bool
    tmp: WorkerTmp
    apps: dict[str, Incomplete]

    def __init__(self, age: int, ppid: int, app_paths: list[str], cfg: Config, log: GLogger, socket_path: str) -> None: ...
    def notify(self) -> None: ...
    def init_process(self) -> None: ...
    def init_signals(self) -> None: ...
    def load_apps(self) -> None: ...
    def run(self) -> None: ...
    async def handle_connection(self, reader: StreamReader, writer: StreamWriter) -> None: ...
    async def handle_request(self, message: dict[str, Incomplete], writer: StreamWriter) -> None: ...
    async def execute(
        self, app_path: str, action: str, args: Iterable[Any], kwargs: Mapping[str, Any]
    ) -> Any: ...  # Arguments and result depend on method name passed to action



import sys

# sys.platform.startswith("aix"):
if sys.platform != "linux" and sys.platform != "win32" and sys.platform != "darwin":
    from typing import Final

    AF_LINK: Final = 18

    def getpagesize() -> int: ...
    def net_if_addrs() -> list[tuple[str, int, str, str | None, str | None, str | None]]: ...
    def net_if_flags(nic_name: str, /) -> list[str]: ...
    def net_if_is_running(nic_name: str, /) -> bool: ...
    def net_if_mtu(nic_name: str, /) -> int: ...
    def proc_priority_get(pid: int, /) -> int: ...
    def proc_priority_set(pid: int, priority: int, /) -> None: ...

    version: Final[int]
    SIDL: Final[int]
    SZOMB: Final[int]
    SACTIVE: Final[int]
    SSWAP: Final[int]
    SSTOP: Final[int]
    TCPS_CLOSED: Final[int]
    TCPS_CLOSING: Final[int]
    TCPS_CLOSE_WAIT: Final[int]
    TCPS_LISTEN: Final[int]
    TCPS_ESTABLISHED: Final[int]
    TCPS_SYN_SENT: Final[int]
    TCPS_SYN_RCVD: Final[int]
    TCPS_FIN_WAIT_1: Final[int]
    TCPS_FIN_WAIT_2: Final[int]
    TCPS_LAST_ACK: Final[int]
    TCPS_TIME_WAIT: Final[int]
    PSUTIL_CONN_NONE: Final = 128

    def proc_args(pid: int, /) -> list[str]: ...
    def proc_basic_info(pid: int, procfs_path: str, /) -> tuple[int, int, int, float, int, int, int, int]: ...
    def proc_cpu_times(pid: int, procfs_path: str, /) -> tuple[float, float, float, float]: ...
    def proc_cred(pid: int, procfs_path: str, /) -> tuple[int, int, int, int, int, int]: ...
    def proc_environ(pid: int, /) -> dict[str, str]: ...
    def proc_name(pid: int, procfs_path: str, /) -> str: ...
    def proc_threads(pid: int, /) -> list[tuple[int, float, float]]: ...
    def proc_io_counters(pid: int, /) -> tuple[int, int, int, int]: ...
    def proc_num_ctx_switches(requested_pid: int, /) -> tuple[int, int]: ...
    def boot_time() -> float: ...
    def disk_io_counters() -> dict[str, tuple[int, int, int, int, int, int]]: ...
    def disk_partitions() -> list[tuple[str, str, str, str]]: ...
    def per_cpu_times() -> list[tuple[float, float, float, float]]: ...
    def swap_mem() -> tuple[int, int, int, int]: ...
    def virtual_mem() -> tuple[int, int, int, int, int]: ...
    def net_io_counters() -> dict[str, tuple[int, int, int, int, int, int, int, int]]: ...
    def cpu_stats() -> tuple[int, int, int, int]: ...
    def net_connections(
        requested_pid: int, /
    ) -> list[tuple[int, int, int, str | tuple[str, int], str | tuple[str, int] | tuple[()], int, int]]: ...
    def net_if_stats(nic_name: str, /) -> tuple[bool, int]: ...  # It's actually list of 2 elements
    def check_pid_range(pid: int, /) -> None: ...
    def set_debug(value: bool, /) -> None: ...



from collections.abc import Callable
from typing import Any, Final, Literal

NO_EDGE: Final = 0
RISING_EDGE: Final = 1
FALLING_EDGE: Final = 2
BOTH_EDGE: Final = 3

def add_edge_detect(
    chip_fd: int, chip_name: str, channel: int, request: int, bouncetime: int, poll_time: float
) -> Literal[1, 2, 0]: ...
def remove_edge_detect(chip_name: str, channel: int, timeout: float = 0.3) -> None: ...
def add_edge_callback(chip_name: str, channel: int, callback: Callable[[int], None]) -> None: ...
def edge_event_detected(chip_name: str, channel: int) -> bool: ...
def gpio_event_added(chip_name: str, channel: int) -> Any: ...
def blocking_wait_for_edge(chip_fd: int, chip_name: str, channel: int, request: int, bouncetime: int, timeout: float) -> int: ...
def event_cleanup(chip_name: str, channel: int) -> None: ...



from _typeshed import Incomplete

from authlib.oauth2.rfc6749 import ClientMixin

class OAuth2ClientMixin(ClientMixin):
    client_id: Incomplete
    client_secret: Incomplete
    client_id_issued_at: Incomplete
    client_secret_expires_at: Incomplete
    _client_metadata: Incomplete
    @property
    def client_info(self) -> dict[str, Incomplete]: ...
    @property
    def client_metadata(self): ...
    def set_client_metadata(self, value) -> None: ...
    @property
    def redirect_uris(self): ...
    @property
    def token_endpoint_auth_method(self): ...
    @property
    def grant_types(self): ...
    @property
    def response_types(self): ...
    @property
    def client_name(self): ...
    @property
    def client_uri(self): ...
    @property
    def logo_uri(self): ...
    @property
    def scope(self): ...
    @property
    def contacts(self): ...
    @property
    def tos_uri(self): ...
    @property
    def policy_uri(self): ...
    @property
    def jwks_uri(self): ...
    @property
    def jwks(self): ...
    @property
    def software_id(self): ...
    @property
    def software_version(self): ...
    @property
    def id_token_signed_response_alg(self): ...
    def get_client_id(self): ...
    def get_default_redirect_uri(self): ...
    def get_allowed_scope(self, scope) -> str: ...
    def check_redirect_uri(self, redirect_uri) -> bool: ...
    def check_client_secret(self, client_secret) -> bool: ...
    def check_endpoint_auth_method(self, method, endpoint) -> bool: ...
    def check_response_type(self, response_type) -> bool: ...
    def check_grant_type(self, grant_type) -> bool: ...



import sys
from _typeshed import AnyStr_co, StrOrBytesPath
from collections.abc import Callable, Iterable
from types import GenericAlias, TracebackType
from typing import IO, Any, AnyStr, Generic, Literal, Protocol, TypeAlias, overload, type_check_only
from typing_extensions import Self, deprecated

__all__ = [
    "input",
    "close",
    "nextfile",
    "filename",
    "lineno",
    "filelineno",
    "fileno",
    "isfirstline",
    "isstdin",
    "FileInput",
    "hook_compressed",
    "hook_encoded",
]

if sys.version_info >= (3, 11):
    _TextMode: TypeAlias = Literal["r"]
else:
    _TextMode: TypeAlias = Literal["r", "rU", "U"]

@type_check_only
class _HasReadlineAndFileno(Protocol[AnyStr_co]):
    def readline(self) -> AnyStr_co: ...
    def fileno(self) -> int: ...

# encoding and errors are added
@overload
def input(
    files: StrOrBytesPath | Iterable[StrOrBytesPath] | None = None,
    inplace: bool = False,
    backup: str = "",
    *,
    mode: _TextMode = "r",
    openhook: Callable[[StrOrBytesPath, str], _HasReadlineAndFileno[str]] | None = None,
    encoding: str | None = None,
    errors: str | None = None,
) -> FileInput[str]: ...
@overload
def input(
    files: StrOrBytesPath | Iterable[StrOrBytesPath] | None = None,
    inplace: bool = False,
    backup: str = "",
    *,
    mode: Literal["rb"],
    openhook: Callable[[StrOrBytesPath, str], _HasReadlineAndFileno[bytes]] | None = None,
    encoding: None = None,
    errors: None = None,
) -> FileInput[bytes]: ...
@overload
def input(
    files: StrOrBytesPath | Iterable[StrOrBytesPath] | None = None,
    inplace: bool = False,
    backup: str = "",
    *,
    mode: str,
    openhook: Callable[[StrOrBytesPath, str], _HasReadlineAndFileno[Any]] | None = None,
    encoding: str | None = None,
    errors: str | None = None,
) -> FileInput[Any]: ...

def close() -> None: ...
def nextfile() -> None: ...
def filename() -> str: ...
def lineno() -> int: ...
def filelineno() -> int: ...
def fileno() -> int: ...
def isfirstline() -> bool: ...
def isstdin() -> bool: ...

class FileInput(Generic[AnyStr]):
    # encoding and errors are added
    @overload
    def __init__(
        self: FileInput[str],
        files: StrOrBytesPath | Iterable[StrOrBytesPath] | None = None,
        inplace: bool = False,
        backup: str = "",
        *,
        mode: _TextMode = "r",
        openhook: Callable[[StrOrBytesPath, str], _HasReadlineAndFileno[str]] | None = None,
        encoding: str | None = None,
        errors: str | None = None,
    ) -> None: ...
    @overload
    def __init__(
        self: FileInput[bytes],
        files: StrOrBytesPath | Iterable[StrOrBytesPath] | None = None,
        inplace: bool = False,
        backup: str = "",
        *,
        mode: Literal["rb"],
        openhook: Callable[[StrOrBytesPath, str], _HasReadlineAndFileno[bytes]] | None = None,
        encoding: None = None,
        errors: None = None,
    ) -> None: ...
    @overload
    def __init__(
        self: FileInput[Any],
        files: StrOrBytesPath | Iterable[StrOrBytesPath] | None = None,
        inplace: bool = False,
        backup: str = "",
        *,
        mode: str,
        openhook: Callable[[StrOrBytesPath, str], _HasReadlineAndFileno[Any]] | None = None,
        encoding: str | None = None,
        errors: str | None = None,
    ) -> None: ...

    def __del__(self) -> None: ...
    def close(self) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self, type: type[BaseException] | None, value: BaseException | None, traceback: TracebackType | None
    ) -> None: ...
    def __iter__(self) -> Self: ...
    def __next__(self) -> AnyStr: ...
    if sys.version_info < (3, 11):
        def __getitem__(self, i: int) -> AnyStr: ...

    def nextfile(self) -> None: ...
    def readline(self) -> AnyStr: ...
    def filename(self) -> str: ...
    def lineno(self) -> int: ...
    def filelineno(self) -> int: ...
    def fileno(self) -> int: ...
    def isfirstline(self) -> bool: ...
    def isstdin(self) -> bool: ...
    def __class_getitem__(cls, item: Any, /) -> GenericAlias: ...

def hook_compressed(
    filename: StrOrBytesPath, mode: str, *, encoding: str | None = None, errors: str | None = None
) -> IO[Any]: ...
@deprecated("Deprecated since Python 3.10. Use `fileinput.input` or `fileinput.FileInput` instead.")
def hook_encoded(encoding: str, errors: str | None = None) -> Callable[[StrOrBytesPath, str], IO[Any]]: ...



import asyncio
from typing import Final

from gunicorn.glogging import Logger as GLogger

from .._types import _ASGIAppType, _ScopeType

OPCODE_CONTINUATION: Final = 0x0
OPCODE_TEXT: Final = 0x1
OPCODE_BINARY: Final = 0x2
OPCODE_CLOSE: Final = 0x8
OPCODE_PING: Final = 0x9
OPCODE_PONG: Final = 0xA
CLOSE_NORMAL: Final = 1000
CLOSE_GOING_AWAY: Final = 1001
CLOSE_PROTOCOL_ERROR: Final = 1002
CLOSE_UNSUPPORTED: Final = 1003
CLOSE_NO_STATUS: Final = 1005
CLOSE_ABNORMAL: Final = 1006
CLOSE_INVALID_DATA: Final = 1007
CLOSE_POLICY_VIOLATION: Final = 1008
CLOSE_MESSAGE_TOO_BIG: Final = 1009
CLOSE_MANDATORY_EXT: Final = 1010
CLOSE_INTERNAL_ERROR: Final = 1011
WS_GUID: Final = b"258EAFA5-E914-47DA-95CA-C5AB0DC85B11"

class WebSocketProtocol:
    transport: asyncio.Transport
    scope: _ScopeType
    app: _ASGIAppType
    log: GLogger
    accepted: bool
    closed: bool
    close_code: int | None
    close_reason: str | None

    def __init__(self, transport: asyncio.Transport, scope: _ScopeType, app: _ASGIAppType, log: GLogger) -> None: ...
    def feed_data(self, data: bytes) -> None: ...
    def feed_eof(self) -> None: ...
    async def run(self) -> None: ...



from _typeshed import ConvertibleToInt, Incomplete, Unused
from re import Pattern
from typing import ClassVar, Final, Literal, SupportsIndex, TypeGuard, overload

from openpyxl.descriptors import Strict, String
from openpyxl.descriptors.base import Integer
from openpyxl.descriptors.serialisable import Serialisable

BUILTIN_FORMATS: Final[dict[int, str]]
BUILTIN_FORMATS_MAX_SIZE: Final = 164
BUILTIN_FORMATS_REVERSE: Final[dict[str, int]]
FORMAT_GENERAL: Final = "General"
FORMAT_TEXT: Final = "@"
FORMAT_NUMBER: Final = "0"
FORMAT_NUMBER_00: Final = "0.00"
FORMAT_NUMBER_COMMA_SEPARATED1: Final = "#,##0.00"
FORMAT_NUMBER_COMMA_SEPARATED2: Final = "#,##0.00_-"
FORMAT_PERCENTAGE: Final = "0%"
FORMAT_PERCENTAGE_00: Final = "0.00%"
FORMAT_DATE_YYYYMMDD2: Final = "yyyy-mm-dd"
FORMAT_DATE_YYMMDD: Final = "yy-mm-dd"
FORMAT_DATE_DDMMYY: Final = "dd/mm/yy"
FORMAT_DATE_DMYSLASH: Final = "d/m/y"
FORMAT_DATE_DMYMINUS: Final = "d-m-y"
FORMAT_DATE_DMMINUS: Final = "d-m"
FORMAT_DATE_MYMINUS: Final = "m-y"
FORMAT_DATE_XLSX14: Final = "mm-dd-yy"
FORMAT_DATE_XLSX15: Final = "d-mmm-yy"
FORMAT_DATE_XLSX16: Final = "d-mmm"
FORMAT_DATE_XLSX17: Final = "mmm-yy"
FORMAT_DATE_XLSX22: Final = "m/d/yy h:mm"
FORMAT_DATE_DATETIME: Final = "yyyy-mm-dd h:mm:ss"
FORMAT_DATE_TIME1: Final = "h:mm AM/PM"
FORMAT_DATE_TIME2: Final = "h:mm:ss AM/PM"
FORMAT_DATE_TIME3: Final = "h:mm"
FORMAT_DATE_TIME4: Final = "h:mm:ss"
FORMAT_DATE_TIME5: Final = "mm:ss"
FORMAT_DATE_TIME6: Final = "h:mm:ss"
FORMAT_DATE_TIME7: Final = "i:s.S"
FORMAT_DATE_TIME8: Final = "h:mm:ss@"
FORMAT_DATE_TIMEDELTA: Final = "[hh]:mm:ss"
FORMAT_DATE_YYMMDDSLASH: Final = "yy/mm/dd@"
FORMAT_CURRENCY_USD_SIMPLE: Final = '"$"#,##0.00_-'
FORMAT_CURRENCY_USD: Final = "$#,##0_-"
FORMAT_CURRENCY_EUR_SIMPLE: Final = "[$EUR ]#,##0.00_-"

COLORS: Final[str]
LITERAL_GROUP: Final = r'".*?"'
LOCALE_GROUP: Final = r"\[(?!hh?\]|mm?\]|ss?\])[^\]]*\]"
STRIP_RE: Final[Pattern[str]]
TIMEDELTA_RE: Final[Pattern[str]]

def is_date_format(fmt: str | None) -> TypeGuard[str]: ...
def is_timedelta_format(fmt: str | None) -> TypeGuard[str]: ...

@overload
def is_datetime(fmt: None) -> None: ...
@overload
def is_datetime(fmt: str) -> Literal["datetime", "date", "time"] | None: ...

def is_builtin(fmt: str | None) -> TypeGuard[str]: ...
def builtin_format_code(index: int) -> str | None: ...

@overload
def builtin_format_id(fmt: None) -> None: ...
@overload
def builtin_format_id(fmt: str) -> int | None: ...

class NumberFormatDescriptor(String[Incomplete]):
    def __set__(self, instance: Serialisable | Strict, value) -> None: ...

class NumberFormat(Serialisable):
    numFmtId: Integer[Literal[False]]
    formatCode: String[Literal[False]]
    def __init__(self, numFmtId: ConvertibleToInt, formatCode: str) -> None: ...

class NumberFormatList(Serialisable):
    # Overwritten by property below
    # count: Integer
    numFmt: Incomplete
    __elements__: ClassVar[tuple[str, ...]]
    __attrs__: ClassVar[tuple[str, ...]]
    def __init__(self, count: Unused = None, numFmt=()) -> None: ...
    @property
    def count(self) -> int: ...
    def __getitem__(self, idx: SupportsIndex) -> NumberFormat: ...



from typing import NamedTuple

__all__ = ["RL_Codecs"]

class StdCodecData(NamedTuple):
    exceptions: dict[int, int | None] | None
    rexceptions: dict[int, int | None] | None

class ExtCodecData(NamedTuple):
    baseName: str
    exceptions: dict[int, int | None] | None
    rexceptions: dict[int, int | None] | None

class RL_Codecs:
    def __init__(self) -> None: ...
    @staticmethod
    def register() -> None: ...
    @staticmethod
    def add_dynamic_codec(name: str, exceptions, rexceptions) -> None: ...
    @staticmethod
    def remove_dynamic_codec(name: str) -> None: ...
    @staticmethod
    def reset_dynamic_codecs() -> None: ...



from _typeshed import Incomplete
from abc import abstractmethod
from typing import ClassVar

from .. import Command

__all__ = ["config_file", "edit_config", "option_base", "setopt"]

def config_file(kind: str = "local"): ...
def edit_config(filename, settings) -> None: ...

class option_base(Command):
    user_options: ClassVar[list[tuple[str, str, str]]]
    boolean_options: ClassVar[list[str]]
    global_config: Incomplete
    user_config: Incomplete
    filename: Incomplete
    def initialize_options(self) -> None: ...
    def finalize_options(self) -> None: ...
    @abstractmethod
    def run(self) -> None: ...

class setopt(option_base):
    description: str
    user_options: ClassVar[list[tuple[str, str, str]]]
    boolean_options: ClassVar[list[str]]
    command: Incomplete
    option: Incomplete
    set_value: Incomplete
    remove: Incomplete
    def initialize_options(self) -> None: ...
    def finalize_options(self) -> None: ...
    def run(self) -> None: ...



import sys
from _typeshed import FileDescriptorLike, Unused
from collections.abc import Sequence
from struct import Struct
from typing import Any, Final, Literal

__all__ = ["ensure_running", "get_inherited_fds", "connect_to_new_process", "set_forkserver_preload"]

MAXFDS_TO_SEND: Final = 256
SIGNED_STRUCT: Final[Struct]

class ForkServer:
    if sys.version_info >= (3, 15):
        def set_forkserver_preload(
            self, modules_names: list[str], *, on_error: Literal["ignore", "warn", "fail"] = "ignore"
        ) -> None: ...
    else:
        def set_forkserver_preload(self, modules_names: list[str]) -> None: ...

    def get_inherited_fds(self) -> list[int] | None: ...
    def connect_to_new_process(self, fds: Sequence[int]) -> tuple[int, int]: ...
    def ensure_running(self) -> None: ...

if sys.version_info >= (3, 15):
    def main(
        listener_fd: int | None,
        alive_r: FileDescriptorLike,
        preload: Sequence[str],
        main_path: str | None = None,
        sys_path: list[str] | None = None,
        *,
        sys_argv: list[str] | None = None,
        authkey_r: int | None = None,
        on_error: str = "ignore",
    ) -> None: ...

elif sys.version_info >= (3, 14):
    # `sys_argv` parameter added in Python 3.14.3
    def main(
        listener_fd: int | None,
        alive_r: FileDescriptorLike,
        preload: Sequence[str],
        main_path: str | None = None,
        sys_path: list[str] | None = None,
        *,
        sys_argv: list[str] | None = None,
        authkey_r: int | None = None,
    ) -> None: ...

elif sys.version_info >= (3, 13):
    # `sys_argv` parameter added in Python 3.13.12
    def main(
        listener_fd: int | None,
        alive_r: FileDescriptorLike,
        preload: Sequence[str],
        main_path: str | None = None,
        sys_path: list[str] | None = None,
        *,
        sys_argv: list[str] | None = None,
    ) -> None: ...

else:
    def main(
        listener_fd: int | None,
        alive_r: FileDescriptorLike,
        preload: Sequence[str],
        main_path: str | None = None,
        sys_path: Unused = None,
    ) -> None: ...

def read_signed(fd: int) -> Any: ...
def write_signed(fd: int, n: int) -> None: ...

_forkserver: ForkServer
ensure_running = _forkserver.ensure_running
get_inherited_fds = _forkserver.get_inherited_fds
connect_to_new_process = _forkserver.connect_to_new_process
set_forkserver_preload = _forkserver.set_forkserver_preload



from _typeshed import Incomplete
from typing import NamedTuple

class _Event(NamedTuple):
    when: Incomplete
    callback: Incomplete
    args: Incomplete
    kwargs: Incomplete

class EventLoop:
    current: Incomplete
    idlers: Incomplete
    inactive: int
    queue: Incomplete
    rpcs: Incomplete
    rpc_results: Incomplete
    def __init__(self) -> None: ...
    def clear(self) -> None: ...
    def insort_event_right(self, event) -> None: ...
    def call_soon(self, callback, *args, **kwargs) -> None: ...
    def queue_call(self, delay, callback, *args, **kwargs) -> None: ...
    def queue_rpc(self, rpc, callback) -> None: ...
    def add_idle(self, callback, *args, **kwargs) -> None: ...
    def run_idle(self): ...
    def run0(self): ...
    def run1(self): ...
    def run(self) -> None: ...



import sys
from _typeshed import SupportsRead, SupportsWrite
from collections.abc import Callable
from typing import Any, Literal

from .decoder import JSONDecodeError as JSONDecodeError, JSONDecoder as JSONDecoder
from .encoder import JSONEncoder as JSONEncoder

__all__ = ["dump", "dumps", "load", "loads", "JSONDecoder", "JSONDecodeError", "JSONEncoder"]

def dumps(
    obj: Any,
    *,
    skipkeys: bool = False,
    ensure_ascii: bool = True,
    check_circular: bool = True,
    allow_nan: bool = True,
    cls: type[JSONEncoder] | None = None,
    indent: None | int | str = None,
    separators: tuple[str, str] | None = None,
    default: Callable[[Any], Any] | None = None,
    sort_keys: bool = False,
    **kwds: Any,
) -> str: ...
def dump(
    obj: Any,
    fp: SupportsWrite[str],
    *,
    skipkeys: bool = False,
    ensure_ascii: bool = True,
    check_circular: bool = True,
    allow_nan: bool = True,
    cls: type[JSONEncoder] | None = None,
    indent: None | int | str = None,
    separators: tuple[str, str] | None = None,
    default: Callable[[Any], Any] | None = None,
    sort_keys: bool = False,
    **kwds: Any,
) -> None: ...

if sys.version_info >= (3, 15):
    def loads(
        s: str | bytes | bytearray,
        *,
        cls: type[JSONDecoder] | None = None,
        object_hook: Callable[[dict[Any, Any]], Any] | None = None,
        parse_float: Callable[[str], Any] | None = None,
        parse_int: Callable[[str], Any] | None = None,
        parse_constant: Callable[[str], Any] | None = None,
        object_pairs_hook: Callable[[list[tuple[Any, Any]]], Any] | None = None,
        array_hook: Callable[[list[Any]], Any] | None = None,
        **kwds: Any,
    ) -> Any: ...
    def load(
        fp: SupportsRead[str | bytes],
        *,
        cls: type[JSONDecoder] | None = None,
        object_hook: Callable[[dict[Any, Any]], Any] | None = None,
        parse_float: Callable[[str], Any] | None = None,
        parse_int: Callable[[str], Any] | None = None,
        parse_constant: Callable[[str], Any] | None = None,
        object_pairs_hook: Callable[[list[tuple[Any, Any]]], Any] | None = None,
        array_hook: Callable[[list[Any]], Any] | None = None,
        **kwds: Any,
    ) -> Any: ...

else:
    def loads(
        s: str | bytes | bytearray,
        *,
        cls: type[JSONDecoder] | None = None,
        object_hook: Callable[[dict[Any, Any]], Any] | None = None,
        parse_float: Callable[[str], Any] | None = None,
        parse_int: Callable[[str], Any] | None = None,
        parse_constant: Callable[[str], Any] | None = None,
        object_pairs_hook: Callable[[list[tuple[Any, Any]]], Any] | None = None,
        **kwds: Any,
    ) -> Any: ...
    def load(
        fp: SupportsRead[str | bytes],
        *,
        cls: type[JSONDecoder] | None = None,
        object_hook: Callable[[dict[Any, Any]], Any] | None = None,
        parse_float: Callable[[str], Any] | None = None,
        parse_int: Callable[[str], Any] | None = None,
        parse_constant: Callable[[str], Any] | None = None,
        object_pairs_hook: Callable[[list[tuple[Any, Any]]], Any] | None = None,
        **kwds: Any,
    ) -> Any: ...

def detect_encoding(
    b: bytes | bytearray,
) -> Literal["utf-8", "utf-8-sig", "utf-16", "utf-16-be", "utf-16-le", "utf-32", "utf-32-be", "utf-32-le"]: ...  # undocumented



from typing import Any, final

from google.protobuf.message import Message

@final
class UnknownFieldSet:
    def __new__(cls, msg: Message) -> UnknownFieldSet: ...  # noqa: Y034
    def __getitem__(self, index: int, /) -> Any: ...  # Any: internal unknown field object
    def __len__(self) -> int: ...



from braintree.graphql.enums import Recommendations as Recommendations, RecommendedPaymentOption as RecommendedPaymentOption
from braintree.graphql.inputs import (
    BillingAddressInput as BillingAddressInput,
    CreateCustomerSessionInput as CreateCustomerSessionInput,
    CreateLocalPaymentContextInput as CreateLocalPaymentContextInput,
    CustomerRecommendationsInput as CustomerRecommendationsInput,
    CustomerSessionInput as CustomerSessionInput,
    MonetaryAmountInput as MonetaryAmountInput,
    PayerInfoInput as PayerInfoInput,
    PayPalPayeeInput as PayPalPayeeInput,
    PayPalPurchaseUnitInput as PayPalPurchaseUnitInput,
    PhoneInput as PhoneInput,
    UpdateCustomerSessionInput as UpdateCustomerSessionInput,
)
from braintree.graphql.types import (
    CustomerRecommendationsPayload as CustomerRecommendationsPayload,
    PaymentOptions as PaymentOptions,
    PaymentRecommendation as PaymentRecommendation,
)
from braintree.graphql.unions import CustomerRecommendations as CustomerRecommendations



from authlib.oauth1 import (
    SIGNATURE_HMAC_SHA1 as SIGNATURE_HMAC_SHA1,
    SIGNATURE_PLAINTEXT as SIGNATURE_PLAINTEXT,
    SIGNATURE_RSA_SHA1 as SIGNATURE_RSA_SHA1,
    SIGNATURE_TYPE_BODY as SIGNATURE_TYPE_BODY,
    SIGNATURE_TYPE_HEADER as SIGNATURE_TYPE_HEADER,
    SIGNATURE_TYPE_QUERY as SIGNATURE_TYPE_QUERY,
)

from ..base_client import OAuthError as OAuthError
from .assertion_session import AssertionSession as AssertionSession
from .oauth1_session import OAuth1Auth as OAuth1Auth, OAuth1Session as OAuth1Session
from .oauth2_session import OAuth2Auth as OAuth2Auth, OAuth2Session as OAuth2Session

__all__ = [
    "OAuthError",
    "OAuth1Session",
    "OAuth1Auth",
    "SIGNATURE_HMAC_SHA1",
    "SIGNATURE_RSA_SHA1",
    "SIGNATURE_PLAINTEXT",
    "SIGNATURE_TYPE_HEADER",
    "SIGNATURE_TYPE_QUERY",
    "SIGNATURE_TYPE_BODY",
    "OAuth2Session",
    "OAuth2Auth",
    "AssertionSession",
]



from collections.abc import Generator, Iterable, Iterator, Mapping
from typing import Generic, TypeVar

_T = TypeVar("_T")

class UnionFind(Generic[_T]):
    parents: Mapping[_T, _T]
    weights: Mapping[_T, int]
    def __init__(self, elements: Iterable[_T] | None = None) -> None: ...
    def __getitem__(self, object: _T) -> _T: ...
    def __iter__(self) -> Iterator[_T]: ...
    def to_sets(self) -> Generator[set[_T]]: ...
    def union(self, *objects: _T) -> None: ...



from _typeshed import Incomplete

from braintree.attribute_getter import AttributeGetter

class LocalPaymentContext(AttributeGetter):
    def __init__(self, attributes: dict[str, Incomplete] | None = None) -> None: ...



from _typeshed import Incomplete

from gunicorn.glogging import Logger as GLogger

from .._types import _ASGIAppType

class LifespanManager:
    app: _ASGIAppType
    logger: GLogger
    state: dict[Incomplete, Incomplete]

    def __init__(self, app: _ASGIAppType, logger: GLogger, state: dict[Incomplete, Incomplete] | None = None) -> None: ...
    async def startup(self) -> None: ...
    async def shutdown(self) -> None: ...



import asyncio
import io
from _typeshed import ReadableBuffer

class AsyncUnreader:
    reader: asyncio.StreamReader
    buf: io.BytesIO
    max_chunk: int

    def __init__(self, reader: asyncio.StreamReader, max_chunk: int = 8192) -> None: ...
    async def read(self, size: int | None = None) -> bytes: ...
    def unread(self, data: ReadableBuffer) -> None: ...
    def has_buffered_data(self) -> bool: ...



import sys
from _typeshed import BytesPath, GenericPath, MaybeNone, StrOrBytesPath, StrPath, SupportsWrite
from collections.abc import Callable, ItemsView, Iterable, Iterator, Mapping, MutableMapping, Sequence
from re import Pattern
from typing import Any, AnyStr, ClassVar, Final, Literal, TypeAlias, TypeVar, overload, type_check_only
from typing_extensions import deprecated

if sys.version_info >= (3, 14):
    __all__ = (
        "NoSectionError",
        "DuplicateOptionError",
        "DuplicateSectionError",
        "NoOptionError",
        "InterpolationError",
        "InterpolationDepthError",
        "InterpolationMissingOptionError",
        "InterpolationSyntaxError",
        "ParsingError",
        "MissingSectionHeaderError",
        "MultilineContinuationError",
        "UnnamedSectionDisabledError",
        "InvalidWriteError",
        "ConfigParser",
        "RawConfigParser",
        "Interpolation",
        "BasicInterpolation",
        "ExtendedInterpolation",
        "SectionProxy",
        "ConverterMapping",
        "DEFAULTSECT",
        "MAX_INTERPOLATION_DEPTH",
        "UNNAMED_SECTION",
    )
elif sys.version_info >= (3, 13):
    __all__ = (
        "NoSectionError",
        "DuplicateOptionError",
        "DuplicateSectionError",
        "NoOptionError",
        "InterpolationError",
        "InterpolationDepthError",
        "InterpolationMissingOptionError",
        "InterpolationSyntaxError",
        "ParsingError",
        "MissingSectionHeaderError",
        "ConfigParser",
        "RawConfigParser",
        "Interpolation",
        "BasicInterpolation",
        "ExtendedInterpolation",
        "SectionProxy",
        "ConverterMapping",
        "DEFAULTSECT",
        "MAX_INTERPOLATION_DEPTH",
        "UNNAMED_SECTION",
        "MultilineContinuationError",
    )
elif sys.version_info >= (3, 12):
    __all__ = (
        "NoSectionError",
        "DuplicateOptionError",
        "DuplicateSectionError",
        "NoOptionError",
        "InterpolationError",
        "InterpolationDepthError",
        "InterpolationMissingOptionError",
        "InterpolationSyntaxError",
        "ParsingError",
        "MissingSectionHeaderError",
        "ConfigParser",
        "RawConfigParser",
        "Interpolation",
        "BasicInterpolation",
        "ExtendedInterpolation",
        "LegacyInterpolation",
        "SectionProxy",
        "ConverterMapping",
        "DEFAULTSECT",
        "MAX_INTERPOLATION_DEPTH",
    )
else:
    __all__ = [
        "NoSectionError",
        "DuplicateOptionError",
        "DuplicateSectionError",
        "NoOptionError",
        "InterpolationError",
        "InterpolationDepthError",
        "InterpolationMissingOptionError",
        "InterpolationSyntaxError",
        "ParsingError",
        "MissingSectionHeaderError",
        "ConfigParser",
        "SafeConfigParser",
        "RawConfigParser",
        "Interpolation",
        "BasicInterpolation",
        "ExtendedInterpolation",
        "LegacyInterpolation",
        "SectionProxy",
        "ConverterMapping",
        "DEFAULTSECT",
        "MAX_INTERPOLATION_DEPTH",
    ]

if sys.version_info >= (3, 13):
    @type_check_only
    class _UNNAMED_SECTION: ...

    UNNAMED_SECTION: _UNNAMED_SECTION

    _SectionName: TypeAlias = str | _UNNAMED_SECTION
    # A list of sections can only include an unnamed section if the parser was initialized with
    # allow_unnamed_section=True. Any prevents users from having to use explicit
    # type checks if allow_unnamed_section is False (the default).
    _SectionNameList: TypeAlias = list[Any]
else:
    _SectionName: TypeAlias = str
    _SectionNameList: TypeAlias = list[str]

_Section: TypeAlias = Mapping[str, str]
_Parser: TypeAlias = MutableMapping[str, _Section]
_ConverterCallback: TypeAlias = Callable[[str], Any]
_ConvertersMap: TypeAlias = dict[str, _ConverterCallback]
_T = TypeVar("_T")

DEFAULTSECT: Final = "DEFAULT"
MAX_INTERPOLATION_DEPTH: Final = 10

class Interpolation:
    def before_get(self, parser: _Parser, section: _SectionName, option: str, value: str, defaults: _Section) -> str: ...
    def before_set(self, parser: _Parser, section: _SectionName, option: str, value: str) -> str: ...
    def before_read(self, parser: _Parser, section: _SectionName, option: str, value: str) -> str: ...
    def before_write(self, parser: _Parser, section: _SectionName, option: str, value: str) -> str: ...

class BasicInterpolation(Interpolation): ...
class ExtendedInterpolation(Interpolation): ...

if sys.version_info < (3, 13):
    @deprecated(
        "Deprecated since Python 3.2; removed in Python 3.13. Use `BasicInterpolation` or `ExtendedInterpolation` instead."
    )
    class LegacyInterpolation(Interpolation):
        def before_get(self, parser: _Parser, section: _SectionName, option: str, value: str, vars: _Section) -> str: ...

class RawConfigParser(_Parser):
    _SECT_TMPL: ClassVar[str]  # undocumented
    _OPT_TMPL: ClassVar[str]  # undocumented
    _OPT_NV_TMPL: ClassVar[str]  # undocumented

    SECTCRE: Pattern[str]
    OPTCRE: ClassVar[Pattern[str]]
    OPTCRE_NV: ClassVar[Pattern[str]]  # undocumented
    NONSPACECRE: ClassVar[Pattern[str]]  # undocumented

    BOOLEAN_STATES: ClassVar[Mapping[str, bool]]  # undocumented
    default_section: str
    if sys.version_info >= (3, 13):
        @overload
        def __init__(
            self,
            defaults: Mapping[str, str | None] | None = None,
            dict_type: type[Mapping[str, str]] = ...,
            *,
            allow_no_value: Literal[True],
            delimiters: Sequence[str] = ("=", ":"),
            comment_prefixes: Sequence[str] = ("#", ";"),
            inline_comment_prefixes: Sequence[str] | None = None,
            strict: bool = True,
            empty_lines_in_values: bool = True,
            default_section: str = "DEFAULT",
            interpolation: Interpolation | None = ...,
            converters: _ConvertersMap = ...,
            allow_unnamed_section: bool = False,
        ) -> None: ...
        @overload
        def __init__(
            self,
            defaults: Mapping[str, str | None] | None,
            dict_type: type[Mapping[str, str]],
            allow_no_value: Literal[True],
            *,
            delimiters: Sequence[str] = ("=", ":"),
            comment_prefixes: Sequence[str] = ("#", ";"),
            inline_comment_prefixes: Sequence[str] | None = None,
            strict: bool = True,
            empty_lines_in_values: bool = True,
            default_section: str = "DEFAULT",
            interpolation: Interpolation | None = ...,
            converters: _ConvertersMap = ...,
            allow_unnamed_section: bool = False,
        ) -> None: ...
        @overload
        def __init__(
            self,
            defaults: _Section | None = None,
            dict_type: type[Mapping[str, str]] = ...,
            allow_no_value: bool = False,
            *,
            delimiters: Sequence[str] = ("=", ":"),
            comment_prefixes: Sequence[str] = ("#", ";"),
            inline_comment_prefixes: Sequence[str] | None = None,
            strict: bool = True,
            empty_lines_in_values: bool = True,
            default_section: str = "DEFAULT",
            interpolation: Interpolation | None = ...,
            converters: _ConvertersMap = ...,
            allow_unnamed_section: bool = False,
        ) -> None: ...
    else:
        @overload
        def __init__(
            self,
            defaults: Mapping[str, str | None] | None = None,
            dict_type: type[Mapping[str, str]] = ...,
            *,
            allow_no_value: Literal[True],
            delimiters: Sequence[str] = ("=", ":"),
            comment_prefixes: Sequence[str] = ("#", ";"),
            inline_comment_prefixes: Sequence[str] | None = None,
            strict: bool = True,
            empty_lines_in_values: bool = True,
            default_section: str = "DEFAULT",
            interpolation: Interpolation | None = ...,
            converters: _ConvertersMap = ...,
        ) -> None: ...
        @overload
        def __init__(
            self,
            defaults: Mapping[str, str | None] | None,
            dict_type: type[Mapping[str, str]],
            allow_no_value: Literal[True],
            *,
            delimiters: Sequence[str] = ("=", ":"),
            comment_prefixes: Sequence[str] = ("#", ";"),
            inline_comment_prefixes: Sequence[str] | None = None,
            strict: bool = True,
            empty_lines_in_values: bool = True,
            default_section: str = "DEFAULT",
            interpolation: Interpolation | None = ...,
            converters: _ConvertersMap = ...,
        ) -> None: ...
        @overload
        def __init__(
            self,
            defaults: _Section | None = None,
            dict_type: type[Mapping[str, str]] = ...,
            allow_no_value: bool = False,
            *,
            delimiters: Sequence[str] = ("=", ":"),
            comment_prefixes: Sequence[str] = ("#", ";"),
            inline_comment_prefixes: Sequence[str] | None = None,
            strict: bool = True,
            empty_lines_in_values: bool = True,
            default_section: str = "DEFAULT",
            interpolation: Interpolation | None = ...,
            converters: _ConvertersMap = ...,
        ) -> None: ...

    def __len__(self) -> int: ...
    def __getitem__(self, key: _SectionName) -> SectionProxy: ...
    def __setitem__(self, key: _SectionName, value: _Section) -> None: ...
    def __delitem__(self, key: _SectionName) -> None: ...
    def __iter__(self) -> Iterator[str]: ...
    def __contains__(self, key: object) -> bool: ...
    def defaults(self) -> _Section: ...
    def sections(self) -> _SectionNameList: ...
    def add_section(self, section: _SectionName) -> None: ...
    def has_section(self, section: _SectionName) -> bool: ...
    def options(self, section: _SectionName) -> list[str]: ...
    def has_option(self, section: _SectionName, option: str) -> bool: ...

    @overload
    def read(self, filenames: GenericPath[AnyStr], encoding: str | None = None) -> list[AnyStr]: ...
    @overload
    def read(self, filenames: Iterable[StrPath], encoding: str | None = None) -> list[str]: ...
    @overload
    def read(self, filenames: Iterable[BytesPath], encoding: str | None = None) -> list[bytes]: ...
    @overload
    def read(self, filenames: Iterable[StrOrBytesPath], encoding: str | None = None) -> list[str | bytes]: ...

    def read_file(self, f: Iterable[str], source: str | None = None) -> None: ...
    def read_string(self, string: str, source: str = "<string>") -> None: ...
    def read_dict(self, dictionary: Mapping[str, Mapping[str, Any]], source: str = "<dict>") -> None: ...
    if sys.version_info < (3, 12):
        @deprecated("Deprecated since Python 3.2; removed in Python 3.12. Use `parser.read_file()` instead.")
        def readfp(self, fp: Iterable[str], filename: str | None = None) -> None: ...

    # These get* methods are partially applied (with the same names) in
    # SectionProxy; the stubs should be kept updated together
    @overload
    def getint(self, section: _SectionName, option: str, *, raw: bool = False, vars: _Section | None = None) -> int: ...
    @overload
    def getint(
        self, section: _SectionName, option: str, *, raw: bool = False, vars: _Section | None = None, fallback: _T
    ) -> int | _T: ...

    @overload
    def getfloat(self, section: _SectionName, option: str, *, raw: bool = False, vars: _Section | None = None) -> float: ...
    @overload
    def getfloat(
        self, section: _SectionName, option: str, *, raw: bool = False, vars: _Section | None = None, fallback: _T
    ) -> float | _T: ...

    @overload
    def getboolean(self, section: _SectionName, option: str, *, raw: bool = False, vars: _Section | None = None) -> bool: ...
    @overload
    def getboolean(
        self, section: _SectionName, option: str, *, raw: bool = False, vars: _Section | None = None, fallback: _T
    ) -> bool | _T: ...

    def _get_conv(
        self,
        section: _SectionName,
        option: str,
        conv: Callable[[str], _T],
        *,
        raw: bool = False,
        vars: _Section | None = None,
        fallback: _T = ...,
    ) -> _T: ...

    # This is incompatible with MutableMapping so we ignore the type
    @overload  # type: ignore[override]
    def get(self, section: _SectionName, option: str, *, raw: bool = False, vars: _Section | None = None) -> str | MaybeNone: ...
    @overload
    def get(
        self, section: _SectionName, option: str, *, raw: bool = False, vars: _Section | None = None, fallback: _T
    ) -> str | _T | MaybeNone: ...

    @overload
    def items(self, *, raw: bool = False, vars: _Section | None = None) -> ItemsView[str, SectionProxy]: ...
    @overload
    def items(self, section: _SectionName, raw: bool = False, vars: _Section | None = None) -> list[tuple[str, str]]: ...

    def set(self, section: _SectionName, option: str, value: str | None = None) -> None: ...
    def write(self, fp: SupportsWrite[str], space_around_delimiters: bool = True) -> None: ...
    def remove_option(self, section: _SectionName, option: str) -> bool: ...
    def remove_section(self, section: _SectionName) -> bool: ...
    def optionxform(self, optionstr: str) -> str: ...
    @property
    def converters(self) -> ConverterMapping: ...

class ConfigParser(RawConfigParser):
    # This is incompatible with MutableMapping so we ignore the type
    @overload  # type: ignore[override]
    def get(self, section: _SectionName, option: str, *, raw: bool = False, vars: _Section | None = None) -> str: ...
    @overload
    def get(
        self, section: _SectionName, option: str, *, raw: bool = False, vars: _Section | None = None, fallback: _T
    ) -> str | _T: ...

if sys.version_info < (3, 12):
    @deprecated("Deprecated since Python 3.2; removed in Python 3.12. Use `ConfigParser` instead.")
    class SafeConfigParser(ConfigParser): ...

class SectionProxy(MutableMapping[str, str]):
    def __init__(self, parser: RawConfigParser, name: str) -> None: ...
    def __getitem__(self, key: str) -> str: ...
    def __setitem__(self, key: str, value: str) -> None: ...
    def __delitem__(self, key: str) -> None: ...
    def __contains__(self, key: object) -> bool: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[str]: ...
    @property
    def parser(self) -> RawConfigParser: ...
    @property
    def name(self) -> str: ...

    # This is incompatible with MutableMapping so we ignore the type
    @overload  # type: ignore[override]
    def get(
        self,
        option: str,
        fallback: None = None,
        *,
        raw: bool = False,
        vars: _Section | None = None,
        _impl: Any | None = None,
        **kwargs: Any,  # passed to the underlying parser's get() method
    ) -> str | None: ...
    @overload
    def get(
        self,
        option: str,
        fallback: _T,
        *,
        raw: bool = False,
        vars: _Section | None = None,
        _impl: Any | None = None,
        **kwargs: Any,  # passed to the underlying parser's get() method
    ) -> str | _T: ...

    # These are partially-applied version of the methods with the same names in
    # RawConfigParser; the stubs should be kept updated together
    @overload
    def getint(self, option: str, *, raw: bool = False, vars: _Section | None = None) -> int | None: ...
    @overload
    def getint(self, option: str, fallback: _T = ..., *, raw: bool = False, vars: _Section | None = None) -> int | _T: ...

    @overload
    def getfloat(self, option: str, *, raw: bool = False, vars: _Section | None = None) -> float | None: ...
    @overload
    def getfloat(self, option: str, fallback: _T = ..., *, raw: bool = False, vars: _Section | None = None) -> float | _T: ...

    @overload
    def getboolean(self, option: str, *, raw: bool = False, vars: _Section | None = None) -> bool | None: ...
    @overload
    def getboolean(self, option: str, fallback: _T = ..., *, raw: bool = False, vars: _Section | None = None) -> bool | _T: ...

    # SectionProxy can have arbitrary attributes when custom converters are used
    def __getattr__(self, key: str) -> Callable[..., Any]: ...

class ConverterMapping(MutableMapping[str, _ConverterCallback | None]):
    GETTERCRE: ClassVar[Pattern[Any]]
    def __init__(self, parser: RawConfigParser) -> None: ...
    def __getitem__(self, key: str) -> _ConverterCallback: ...
    def __setitem__(self, key: str, value: _ConverterCallback | None) -> None: ...
    def __delitem__(self, key: str) -> None: ...
    def __iter__(self) -> Iterator[str]: ...
    def __len__(self) -> int: ...

class Error(Exception):
    message: str
    def __init__(self, msg: str = "") -> None: ...

class NoSectionError(Error):
    section: _SectionName
    def __init__(self, section: _SectionName) -> None: ...

class DuplicateSectionError(Error):
    section: _SectionName
    source: str | None
    lineno: int | None
    def __init__(self, section: _SectionName, source: str | None = None, lineno: int | None = None) -> None: ...

class DuplicateOptionError(Error):
    section: _SectionName
    option: str
    source: str | None
    lineno: int | None
    def __init__(self, section: _SectionName, option: str, source: str | None = None, lineno: int | None = None) -> None: ...

class NoOptionError(Error):
    section: _SectionName
    option: str
    def __init__(self, option: str, section: _SectionName) -> None: ...

class InterpolationError(Error):
    section: _SectionName
    option: str
    def __init__(self, option: str, section: _SectionName, msg: str) -> None: ...

class InterpolationDepthError(InterpolationError):
    def __init__(self, option: str, section: _SectionName, rawval: object) -> None: ...

class InterpolationMissingOptionError(InterpolationError):
    reference: str
    def __init__(self, option: str, section: _SectionName, rawval: object, reference: str) -> None: ...

class InterpolationSyntaxError(InterpolationError): ...

class ParsingError(Error):
    source: str
    errors: list[tuple[int, str]]
    if sys.version_info >= (3, 13):
        def __init__(self, source: str, *args: object) -> None: ...
        def combine(self, others: Iterable[ParsingError]) -> ParsingError: ...
    elif sys.version_info >= (3, 12):
        def __init__(self, source: str) -> None: ...
    else:
        @overload
        def __init__(self, source: str) -> None: ...
        @overload
        @deprecated("The `filename` parameter removed in Python 3.12. Use `source` instead.")
        def __init__(self, source: None, filename: str | None) -> None: ...
        @overload
        @deprecated("The `filename` parameter removed in Python 3.12. Use `source` instead.")
        def __init__(self, source: None = None, *, filename: str | None) -> None: ...

    def append(self, lineno: int, line: str) -> None: ...

    if sys.version_info < (3, 12):
        @property
        @deprecated("Deprecated since Python 3.2; removed in Python 3.12. Use `source` instead.")
        def filename(self) -> str: ...
        @filename.setter
        @deprecated("Deprecated since Python 3.2; removed in Python 3.12. Use `source` instead.")
        def filename(self, value: str) -> None: ...

class MissingSectionHeaderError(ParsingError):
    lineno: int
    line: str
    def __init__(self, filename: str, lineno: int, line: str) -> None: ...

if sys.version_info >= (3, 13):
    class MultilineContinuationError(ParsingError):
        lineno: int
        line: str
        def __init__(self, filename: str, lineno: int, line: str) -> None: ...

if sys.version_info >= (3, 14):
    class UnnamedSectionDisabledError(Error):
        msg: Final = "Support for UNNAMED_SECTION is disabled."
        def __init__(self) -> None: ...

    class InvalidWriteError(Error): ...



from authlib.oauth2.rfc7009 import RevocationEndpoint as _RevocationEndpoint

class RevocationEndpoint(_RevocationEndpoint):
    def query_token(self, token, token_type_hint): ...
    def revoke_token(self, token, request) -> None: ...



from _typeshed import Unused
from typing import IO, Any, Final, Literal, NamedTuple, NoReturn, TypeAlias, overload
from typing_extensions import Self

_File: TypeAlias = str | IO[bytes]

class Error(Exception): ...

AUDIO_FILE_MAGIC: Final = 0x2E736E64
AUDIO_FILE_ENCODING_MULAW_8: Final = 1
AUDIO_FILE_ENCODING_LINEAR_8: Final = 2
AUDIO_FILE_ENCODING_LINEAR_16: Final = 3
AUDIO_FILE_ENCODING_LINEAR_24: Final = 4
AUDIO_FILE_ENCODING_LINEAR_32: Final = 5
AUDIO_FILE_ENCODING_FLOAT: Final = 6
AUDIO_FILE_ENCODING_DOUBLE: Final = 7
AUDIO_FILE_ENCODING_ADPCM_G721: Final = 23
AUDIO_FILE_ENCODING_ADPCM_G722: Final = 24
AUDIO_FILE_ENCODING_ADPCM_G723_3: Final = 25
AUDIO_FILE_ENCODING_ADPCM_G723_5: Final = 26
AUDIO_FILE_ENCODING_ALAW_8: Final = 27
AUDIO_UNKNOWN_SIZE: Final = 0xFFFFFFFF

class _sunau_params(NamedTuple):
    nchannels: int
    sampwidth: int
    framerate: int
    nframes: int
    comptype: str
    compname: str

class Au_read:
    def __init__(self, f: _File) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(self, *args: Unused) -> None: ...
    def __del__(self) -> None: ...
    def getfp(self) -> IO[bytes] | None: ...
    def rewind(self) -> None: ...
    def close(self) -> None: ...
    def tell(self) -> int: ...
    def getnchannels(self) -> int: ...
    def getnframes(self) -> int: ...
    def getsampwidth(self) -> int: ...
    def getframerate(self) -> int: ...
    def getcomptype(self) -> str: ...
    def getcompname(self) -> str: ...
    def getparams(self) -> _sunau_params: ...
    def getmarkers(self) -> None: ...
    def getmark(self, id: Any) -> NoReturn: ...
    def setpos(self, pos: int) -> None: ...
    def readframes(self, nframes: int) -> bytes | None: ...

class Au_write:
    def __init__(self, f: _File) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(self, *args: Unused) -> None: ...
    def __del__(self) -> None: ...
    def setnchannels(self, nchannels: int) -> None: ...
    def getnchannels(self) -> int: ...
    def setsampwidth(self, sampwidth: int) -> None: ...
    def getsampwidth(self) -> int: ...
    def setframerate(self, framerate: float) -> None: ...
    def getframerate(self) -> int: ...
    def setnframes(self, nframes: int) -> None: ...
    def getnframes(self) -> int: ...
    def setcomptype(self, type: str, name: str) -> None: ...
    def getcomptype(self) -> str: ...
    def getcompname(self) -> str: ...
    def setparams(self, params: _sunau_params) -> None: ...
    def getparams(self) -> _sunau_params: ...
    def tell(self) -> int: ...
    # should be any bytes-like object after 3.4, but we don't have a type for that
    def writeframesraw(self, data: bytes) -> None: ...
    def writeframes(self, data: bytes) -> None: ...
    def close(self) -> None: ...

@overload
def open(f: _File, mode: Literal["r", "rb"]) -> Au_read: ...
@overload
def open(f: _File, mode: Literal["w", "wb"]) -> Au_write: ...
@overload
def open(f: _File, mode: str | None = None) -> Any: ...



from ..base_client import BaseApp
from ..base_client.async_app import AsyncOAuth1Mixin, AsyncOAuth2Mixin
from ..base_client.async_openid import AsyncOpenIDMixin
from ..httpx_client import AsyncOAuth1Client, AsyncOAuth2Client

class StarletteAppMixin:
    async def save_authorize_data(self, request, **kwargs) -> None: ...
    async def authorize_redirect(self, request, redirect_uri=None, **kwargs): ...

class StarletteOAuth1App(StarletteAppMixin, AsyncOAuth1Mixin, BaseApp):
    client_cls = AsyncOAuth1Client
    async def authorize_access_token(self, request, **kwargs): ...

class StarletteOAuth2App(StarletteAppMixin, AsyncOAuth2Mixin, AsyncOpenIDMixin, BaseApp):
    client_cls = AsyncOAuth2Client
    async def authorize_access_token(self, request, **kwargs): ...



from _typeshed import Self
from collections.abc import Iterator, Mapping, Sequence, Set as AbstractSet
from typing import Any, ClassVar, TypedDict, type_check_only

from pygments.token import _TokenType

ansicolors: AbstractSet[str]  # not intended to be mutable

@type_check_only
class _StyleDict(TypedDict):
    color: str | None
    bold: bool
    italic: bool
    underline: bool
    bgcolor: str | None
    border: str | None
    roman: bool | None  # lol yes, can be True or False or None
    sans: bool | None
    mono: bool | None
    ansicolor: str | None
    bgansicolor: str | None

class StyleMeta(type):
    def __new__(cls: type[Self], name: str, bases: tuple[type[Any], ...], dct: dict[str, Any]) -> Self: ...
    def style_for_token(cls, token: _TokenType) -> _StyleDict: ...
    def styles_token(cls, ttype: _TokenType) -> bool: ...
    def list_styles(cls) -> list[tuple[_TokenType, _StyleDict]]: ...
    def __iter__(cls) -> Iterator[tuple[_TokenType, _StyleDict]]: ...
    def __len__(cls) -> int: ...

class Style(metaclass=StyleMeta):
    background_color: ClassVar[str]
    highlight_color: ClassVar[str]
    line_number_color: ClassVar[str]
    line_number_background_color: ClassVar[str]
    line_number_special_color: ClassVar[str]
    line_number_special_background_color: ClassVar[str]
    styles: ClassVar[Mapping[_TokenType, str]]  # not intended to be mutable
    name: ClassVar[str]
    aliases: ClassVar[Sequence[str]]  # not intended to be mutable
    web_style_gallery_exclude: ClassVar[bool]



from collections.abc import Iterable
from typing import Literal

from . import color4
from .ast import Node

COLOR_SCHEMES: set[str]
COLOR_SPACES: set[str]
D50: tuple[float, float, float]
D65: tuple[float, float, float]

class Color(color4.Color):
    COLOR_SPACES: set[str] | None

def parse_color(
    input: str | Iterable[Node], color_schemes: Literal["normal"] | Iterable[str] | None = None
) -> color4.Color | Color | Literal["currentcolor"] | None: ...



from _typeshed import FileDescriptorOrPath
from typing import Final

__version__: Final[str]
PLAIN: Final = 1
PREFORMATTED: Final = 2
BULLETCHAR: Final = "\267"

class BaseParser:
    def __init__(self) -> None: ...
    def reset(self) -> None: ...
    def parseFile(self, filename: FileDescriptorOrPath) -> list[tuple[str, str] | tuple[str, str, str]]: ...
    def parseText(self, textBlock: str) -> list[tuple[str, str] | tuple[str, str, str]]: ...
    def readLine(self, line: str) -> None: ...
    def endPara(self) -> None: ...
    def beginPre(self, stylename: str) -> None: ...
    def endPre(self) -> None: ...
    def image(self, filename: str) -> None: ...

class Parser(BaseParser):
    def vSpace(self, points) -> None: ...
    def pageBreak(self) -> None: ...
    def custom(self, moduleName: str, funcName: str) -> None: ...
    def nextPageTemplate(self, templateName: str) -> None: ...

def parseFile(filename: FileDescriptorOrPath) -> list[tuple[str, str] | tuple[str, str, str]]: ...
def parseText(textBlock: str) -> list[tuple[str, str] | tuple[str, str, str]]: ...



from _typeshed import Incomplete, ReadableBuffer
from collections.abc import Iterator
from typing import Literal

from gunicorn.config import Config
from gunicorn.http2.stream import HTTP2Stream

from .._types import _AddressType

class HTTP2Body:
    def __init__(self, data: ReadableBuffer) -> None: ...
    def read(self, size: int | None = None) -> bytes: ...
    def readline(self, size: int | None = None) -> bytes: ...
    def readlines(self, hint: int | None = None) -> list[bytes]: ...
    def __iter__(self) -> Iterator[bytes]: ...
    def __len__(self) -> int: ...
    def close(self) -> None: ...

class HTTP2Request:
    stream: HTTP2Stream
    cfg: Config
    peer_addr: _AddressType
    remote_addr: _AddressType
    version: tuple[int, int]
    method: str
    scheme: Literal["https", "http"]
    uri: str
    path: str
    query: str
    fragment: str
    headers: list[tuple[str, str]]
    trailers: list[tuple[str, Incomplete]]
    body: HTTP2Body
    must_close: bool
    req_number: int
    proxy_protocol_info: dict[str, str | int | None] | None  # TODO: Use TypedDict
    priority_weight: int
    priority_depends_on: int

    def __init__(self, stream: HTTP2Stream, cfg: Config, peer_addr: _AddressType) -> None: ...
    def force_close(self) -> None: ...
    def should_close(self) -> bool: ...
    def get_header(self, name: str) -> str | None: ...
    @property
    def content_length(self) -> int | None: ...
    @property
    def content_type(self) -> str | None: ...

__all__ = ["HTTP2Request", "HTTP2Body"]



from typing import Literal

from reportlab.pdfbase.pdfdoc import PDFDictionary, PDFObject, PDFStream, PDFString
from reportlab.pdfbase.pdfpattern import PDFPattern, PDFPatternIf

def textFieldAbsolute(
    canvas, title, x, y, width, height, value: str = "", maxlen: int = 1000000, multiline: bool | Literal[0, 1] = 0
) -> None: ...
def textFieldRelative(
    canvas, title, xR, yR, width, height, value: str = "", maxlen: int = 1000000, multiline: bool | Literal[0, 1] = 0
) -> None: ...
def buttonFieldAbsolute(canvas, title, value, x, y, width: float = 16.7704, height: float = 14.907) -> None: ...
def buttonFieldRelative(canvas, title, value, xR, yR, width: float = 16.7704, height: float = 14.907) -> None: ...
def selectFieldAbsolute(canvas, title, value, options, x, y, width, height) -> None: ...
def selectFieldRelative(canvas, title, value, options, xR, yR, width, height) -> None: ...
def getForm(canvas) -> AcroForm: ...

class AcroForm(PDFObject):
    fields: list[PDFPattern]
    def __init__(self) -> None: ...
    def textField(
        self, canvas, title, xmin, ymin, xmax, ymax, value: str = "", maxlen: int = 1000000, multiline: bool | Literal[0, 1] = 0
    ) -> None: ...
    def selectField(self, canvas, title, value, options, xmin, ymin, xmax, ymax) -> None: ...
    def buttonField(self, canvas, title, value, xmin, ymin, width: float = 16.7704, height: float = 14.907) -> None: ...
    def format(self, document) -> bytes: ...

FormPattern: list[str | list[str] | PDFString | PDFPatternIf]

def FormFontsDictionary() -> PDFDictionary: ...
def FormResources() -> PDFPattern: ...

ZaDbPattern: list[str]
FormResourcesDictionaryPattern: list[str | list[str]]
FORMFONTNAMES: dict[str, str]
EncodingPattern: list[str | list[str]]
PDFDocEncodingPattern: list[str]

def FormFont(BaseFont, Name) -> PDFPattern: ...

FormFontPattern: list[str | list[str]]

def resetPdfForm() -> None: ...
def TextField(
    title,
    value,
    xmin,
    ymin,
    xmax,
    ymax,
    page,
    maxlen: int = 1000000,
    font: str = "Helvetica-Bold",
    fontsize: int = 9,
    R: int = 0,
    G: int = 0,
    B: float = 0.627,
    multiline: bool | Literal[0, 1] = 0,
) -> PDFPattern: ...

TextFieldPattern: list[str | list[str]]

def SelectField(
    title,
    value,
    options,
    xmin,
    ymin,
    xmax,
    ymax,
    page,
    font: str = "Helvetica-Bold",
    fontsize: int = 9,
    R: int = 0,
    G: int = 0,
    B: float = 0.627,
) -> PDFPattern: ...

SelectFieldPattern: list[str | list[str]]

def ButtonField(title, value, xmin, ymin, page, width: float = 16.7704, height: float = 14.907) -> PDFPattern: ...

ButtonFieldPattern: list[str | list[str] | PDFString]

def buttonStreamDictionary(width: float = 16.7704, height: float = 14.907) -> PDFDictionary: ...
def ButtonStream(content, width: float = 16.7704, height: float = 14.907) -> PDFStream: ...



from _typeshed import Incomplete

from authlib.oauth2.rfc7521 import AssertionClient as _AssertionClient

from ..base_client import OAuthError
from .oauth2_client import OAuth2Auth

__all__ = ["AsyncAssertionClient"]

# Inherits from httpx.AsyncClient
class AsyncAssertionClient(_AssertionClient):
    token_auth_class = OAuth2Auth
    oauth_error_class = OAuthError  # type: ignore[assignment]
    JWT_BEARER_GRANT_TYPE: Incomplete
    ASSERTION_METHODS: Incomplete
    DEFAULT_GRANT_TYPE: Incomplete
    def __init__(
        self,
        token_endpoint,
        issuer,
        subject,
        audience=None,
        grant_type=None,
        claims=None,
        token_placement="header",
        scope=None,
        **kwargs,
    ) -> None: ...
    async def request(self, method, url, withhold_token=False, auth=..., **kwargs): ...

# Inherits from httpx.Client
class AssertionClient(_AssertionClient):
    token_auth_class = OAuth2Auth
    oauth_error_class = OAuthError  # type: ignore[assignment]
    JWT_BEARER_GRANT_TYPE: Incomplete
    ASSERTION_METHODS: Incomplete
    DEFAULT_GRANT_TYPE: Incomplete
    def __init__(
        self,
        token_endpoint,
        issuer,
        subject,
        audience=None,
        grant_type=None,
        claims=None,
        token_placement="header",
        scope=None,
        **kwargs,
    ) -> None: ...
    def request(self, method, url, withhold_token=False, auth=..., **kwargs): ...



import argparse
from collections.abc import Sequence

def main_inner(parser: argparse.ArgumentParser, argns: argparse.Namespace) -> int: ...

class HelpFormatter(argparse.HelpFormatter):
    def __init__(self, prog: str, indent_increment: int = 2, max_help_position: int = 16, width: int | None = None) -> None: ...

def main(args: Sequence[str] | None = ...) -> int: ...



from collections.abc import Callable, Sequence
from typing import Literal

from .constants import *

model = ...
JETSON_INFO = ...
RPI_INFO = ...

def setwarnings(state: bool) -> None: ...
def setmode(mode: Literal[10, 11, 1000, 1001]) -> None: ...
def getmode() -> Literal[10, 11, 1000, 1001]: ...
def setup(
    channels: int | Sequence[int],
    direction: Literal[0, 1],
    pull_up_down: Literal[20, 21, 22] = ...,
    initial: Literal[0, 1] = ...,
    consumer: str = ...,
) -> None: ...
def cleanup(channel: int | Sequence[int] | None = ...) -> None: ...
def input(channel: int) -> Literal[0, 1]: ...
def output(channels: int | Sequence[int], values: Literal[0, 1]) -> None: ...
def add_event_detect(
    channel: int,
    edge: Literal[31, 32, 33],
    callback: Callable[[int], None] | None = ...,
    bouncetime: int | None = ...,
    polltime: float = 0.2,
) -> None: ...
def remove_event_detect(channel: int, timeout: float = 0.5) -> None: ...
def event_detected(channel: int) -> bool: ...
def add_event_callback(channel: int, callback: Callable[[int], None]) -> None: ...
def wait_for_edge(
    channel: int, edge: Literal[31, 32, 33], bouncetime: int | None = ..., timeout: float | None = ...
) -> int | None: ...
def gpio_function(channel: int) -> Literal[-1, 0, 1]: ...

class PWM:
    def __init__(self, channel: int, frequency_hz: float) -> None: ...
    def __del__(self) -> None: ...
    def start(self, duty_cycle_percent: float) -> None: ...
    def ChangeFrequency(self, frequency_hz: float) -> None: ...
    def ChangeDutyCycle(self, duty_cycle_percent: float) -> None: ...
    def stop(self) -> None: ...



from _typeshed import Incomplete

def add_to_uri(token, uri) -> str: ...
def add_to_headers(token, headers=None): ...
def add_to_body(token, body=None) -> str: ...
def add_bearer_token(token, uri, headers, body, placement: str = "header") -> tuple[Incomplete, Incomplete, Incomplete]: ...



from authlib.oauth1.rfc5849.base_server import BaseServer

from .wrapper import OAuth1Request

class ResourceProtector(BaseServer):
    def validate_request(self, method, uri, body, headers) -> OAuth1Request: ...
    def get_token_credential(self, request): ...



from ..base_client import BaseApp, OAuth1Mixin, OAuth2Mixin, OpenIDMixin
from ..requests_client import OAuth1Session, OAuth2Session

class FlaskAppMixin:
    @property
    def token(self): ...
    @token.setter
    def token(self, token): ...

    def save_authorize_data(self, **kwargs) -> None: ...
    def authorize_redirect(self, redirect_uri=None, **kwargs): ...

class FlaskOAuth1App(FlaskAppMixin, OAuth1Mixin, BaseApp):
    client_cls = OAuth1Session
    def authorize_access_token(self, **kwargs): ...

class FlaskOAuth2App(FlaskAppMixin, OAuth2Mixin, OpenIDMixin, BaseApp):
    client_cls = OAuth2Session
    def authorize_access_token(self, **kwargs): ...



from _typeshed import Incomplete
from typing import NoReturn

# Inherits from werkzeug.exceptions.HTTPException
class _HTTPException:
    code: Incomplete
    body: Incomplete
    headers: Incomplete
    def __init__(self, code, body, headers, response=None) -> None: ...
    # Params depends on `werkzeug` package version
    def get_body(self, environ=None, scope=None): ...
    def get_headers(self, environ=None, scope=None): ...

def raise_http_exception(status, body, headers) -> NoReturn: ...



from collections.abc import Callable
from typing import TypeAlias

import numpy as np

__all__ = ["get_filter", "clear_cache", "sinc_window"]

# Dictionary to cache loaded filters
FILTER_CACHE: dict[str, tuple[np.ndarray[tuple[int], np.dtype[np.float64]], int, float]]

# List of filter functions available
FILTER_FUNCTIONS: list[str]

_FilterType: TypeAlias = str | Callable[[int], np.ndarray[tuple[int], np.dtype[np.float64]]]

def sinc_window(
    num_zeros: int = 64,
    precision: int = 9,
    window: Callable[[int], np.ndarray[tuple[int], np.dtype[np.float64]]] | None = None,
    rolloff: float = 0.945,
) -> tuple[np.ndarray[tuple[int], np.dtype[np.float64]], int, float]: ...
def get_filter(
    name_or_function: _FilterType, *, num_zeros: int = 64, precision: int = 9, rolloff: float = 0.945
) -> tuple[np.ndarray[tuple[int], np.dtype[np.float64]], int, float]: ...
def load_filter(filter_name: str) -> tuple[np.ndarray[tuple[int], np.dtype[np.float64]], int, float]: ...
def clear_cache() -> None: ...



from _typeshed import Incomplete

POOLING_STRATEGIES: Incomplete

class ServerState:
    server: Incomplete
    last_checked_time: Incomplete
    available: Incomplete
    def __init__(self, server, last_checked_time, available) -> None: ...

class ServerPoolState:
    server_states: Incomplete
    strategy: Incomplete
    server_pool: Incomplete
    last_used_server: int
    initialize_time: Incomplete
    def __init__(self, server_pool) -> None: ...
    def refresh(self) -> None: ...
    def get_current_server(self): ...
    def get_server(self): ...
    def find_active_random_server(self): ...
    def find_active_server(self, starting): ...
    def __len__(self) -> int: ...

class ServerPool:
    servers: Incomplete
    pool_states: Incomplete
    active: Incomplete
    exhaust: Incomplete
    single: Incomplete
    strategy: Incomplete
    def __init__(
        self, servers=None, pool_strategy="ROUND_ROBIN", active: bool = True, exhaust: bool = False, single_state: bool = True
    ) -> None: ...
    def __len__(self) -> int: ...
    def __getitem__(self, item): ...
    def __iter__(self): ...
    def add(self, servers) -> None: ...
    def remove(self, server) -> None: ...
    def initialize(self, connection) -> None: ...
    def get_server(self, connection): ...
    def get_current_server(self, connection): ...
